@@ -3,15 +3,13 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
-"""mmCIF parsers"""
+"""mmCIF parsers."""
 
-from __future__ import print_function
 
 import numpy
 import warnings
 
 from Bio.File import as_handle
-from Bio._py3k import range
 
 from Bio.PDB.MMCIF2Dict import MMCIF2Dict
 from Bio.PDB.StructureBuilder import StructureBuilder
@@ -19,7 +17,7 @@ from Bio.PDB.PDBExceptions import PDBConstructionException
 from Bio.PDB.PDBExceptions import PDBConstructionWarning
 
 
-class MMCIFParser(object):
+class MMCIFParser:
     """Parse a mmCIF file and return a Structure object."""
 
     def __init__(self, structure_builder=None, QUIET=False):
@@ -35,12 +33,13 @@ class MMCIFParser(object):
          - QUIET - Evaluated as a Boolean. If true, warnings issued in constructing
            the SMCRA data will be suppressed. If false (DEFAULT), they will be shown.
            These warnings might be indicative of problems in the mmCIF file!
+
         """
         if structure_builder is not None:
             self._structure_builder = structure_builder
         else:
             self._structure_builder = StructureBuilder()
-        # self.header = None
+        self.header = None
         # self.trailer = None
         self.line_counter = 0
         self.build_structure = None
@@ -53,19 +52,74 @@ class MMCIFParser(object):
 
         Arguments:
          - structure_id - string, the id that will be used for the structure
-         - filename - name of the mmCIF file OR an open filehandle
+         - filename - name of mmCIF file, OR an open text mode file handle
+
         """
         with warnings.catch_warnings():
             if self.QUIET:
                 warnings.filterwarnings("ignore", category=PDBConstructionWarning)
             self._mmcif_dict = MMCIF2Dict(filename)
             self._build_structure(structure_id)
+            self._structure_builder.set_header(self._get_header())
 
         return self._structure_builder.get_structure()
 
     # Private methods
 
+    def _mmcif_get(self, key, dict, deflt):
+        if key in dict:
+            rslt = dict[key][0]
+            if "?" != rslt:
+                return rslt
+        return deflt
+
+    def _update_header_entry(self, target_key, keys):
+        md = self._mmcif_dict
+        for key in keys:
+            val = md.get(key)
+            try:
+                item = val[0]
+            except (TypeError, IndexError):
+                continue
+            if item != "?":
+                self.header[target_key] = item
+                break
+
+    def _get_header(self):
+        self.header = {
+            "name": "",
+            "head": "",
+            "idcode": "",
+            "deposition_date": "",
+            "structure_method": "",
+            "resolution": 0.0,
+        }
+
+        self._update_header_entry(
+            "idcode", ["_entry_id", "_exptl.entry_id", "_struct.entry_id"]
+        )
+        self._update_header_entry("name", ["_struct.title"])
+        self._update_header_entry(
+            "head", ["_struct_keywords.pdbx_keywords", "_struct_keywords.text"]
+        )
+        self._update_header_entry(
+            "deposition_date", ["_pdbx_database_status.recvd_initial_deposition_date"]
+        )
+        self._update_header_entry("structure_method", ["_exptl.method"])
+        self._update_header_entry(
+            "resolution", ["_refine.ls_d_res_high", "_refine_hist.d_res_high"]
+        )
+        self.header["resolution"] = float(self.header["resolution"])
+
+        return self.header
+
     def _build_structure(self, structure_id):
+
+        # two special chars as placeholders in the mmCIF format
+        # for item values that cannot be explicitly assigned
+        # see: pdbx/mmcif syntax web page
+        _unassigned = {".", "?"}
+
         mmcif_dict = self._mmcif_dict
         atom_id_list = mmcif_dict["_atom_site.label_atom_id"]
         residue_id_list = mmcif_dict["_atom_site.label_comp_id"]
@@ -73,7 +127,6 @@ class MMCIFParser(object):
             element_list = mmcif_dict["_atom_site.type_symbol"]
         except KeyError:
             element_list = None
-        seq_id_list = mmcif_dict["_atom_site.label_seq_id"]
         chain_id_list = mmcif_dict["_atom_site.auth_asym_id"]
         x_list = [float(x) for x in mmcif_dict["_atom_site.Cartn_x"]]
         y_list = [float(x) for x in mmcif_dict["_atom_site.Cartn_y"]]
@@ -90,14 +143,14 @@ class MMCIFParser(object):
             serial_list = None
         except ValueError:
             # Invalid model number (malformed file)
-            raise PDBConstructionException("Invalid model number")
+            raise PDBConstructionException("Invalid model number") from None
         try:
-            aniso_u11 = mmcif_dict["_atom_site.aniso_U[1][1]"]
-            aniso_u12 = mmcif_dict["_atom_site.aniso_U[1][2]"]
-            aniso_u13 = mmcif_dict["_atom_site.aniso_U[1][3]"]
-            aniso_u22 = mmcif_dict["_atom_site.aniso_U[2][2]"]
-            aniso_u23 = mmcif_dict["_atom_site.aniso_U[2][3]"]
-            aniso_u33 = mmcif_dict["_atom_site.aniso_U[3][3]"]
+            aniso_u11 = mmcif_dict["_atom_site_anisotrop.U[1][1]"]
+            aniso_u12 = mmcif_dict["_atom_site_anisotrop.U[1][2]"]
+            aniso_u13 = mmcif_dict["_atom_site_anisotrop.U[1][3]"]
+            aniso_u22 = mmcif_dict["_atom_site_anisotrop.U[2][2]"]
+            aniso_u23 = mmcif_dict["_atom_site_anisotrop.U[2][3]"]
+            aniso_u33 = mmcif_dict["_atom_site_anisotrop.U[3][3]"]
             aniso_flag = 1
         except KeyError:
             # no anisotropic B factors
@@ -132,25 +185,28 @@ class MMCIFParser(object):
             resname = residue_id_list[i]
             chainid = chain_id_list[i]
             altloc = alt_list[i]
-            if altloc == ".":
+            if altloc in _unassigned:
                 altloc = " "
             int_resseq = int(seq_id_list[i])
             icode = icode_list[i]
-            if icode == "?":
+            if icode in _unassigned:
                 icode = " "
             name = atom_id_list[i]
             # occupancy & B factor
             try:
                 tempfactor = float(b_factor_list[i])
             except ValueError:
-                raise PDBConstructionException("Invalid or missing B factor")
+                raise PDBConstructionException("Invalid or missing B factor") from None
             try:
                 occupancy = float(occupancy_list[i])
             except ValueError:
-                raise PDBConstructionException("Invalid or missing occupancy")
+                raise PDBConstructionException("Invalid or missing occupancy") from None
             fieldname = fieldname_list[i]
             if fieldname == "HETATM":
-                hetatm_flag = "H"
+                if resname == "HOH" or resname == "WAT":
+                    hetatm_flag = "W"
+                else:
+                    hetatm_flag = "H"
             else:
                 hetatm_flag = " "
 
@@ -182,35 +238,42 @@ class MMCIFParser(object):
                 current_resname = resname
                 structure_builder.init_residue(resname, hetatm_flag, int_resseq, icode)
 
-            coord = numpy.array((x, y, z), 'f')
-            element = element_list[i] if element_list else None
-            structure_builder.init_atom(name, coord, tempfactor, occupancy, altloc,
-                name, element=element)
-            if aniso_flag == 1:
-                u = (aniso_u11[i], aniso_u12[i], aniso_u13[i],
-                    aniso_u22[i], aniso_u23[i], aniso_u33[i])
-                mapped_anisou = [float(x) for x in u]
-                anisou_array = numpy.array(mapped_anisou, 'f')
+            coord = numpy.array((x, y, z), "f")
+            element = element_list[i].upper() if element_list else None
+            structure_builder.init_atom(
+                name, coord, tempfactor, occupancy, altloc, name, element=element
+            )
+            if aniso_flag == 1 and i < len(aniso_u11):
+                u = (
+                    aniso_u11[i],
+                    aniso_u12[i],
+                    aniso_u13[i],
+                    aniso_u22[i],
+                    aniso_u23[i],
+                    aniso_u33[i],
+                )
+                mapped_anisou = [float(_) for _ in u]
+                anisou_array = numpy.array(mapped_anisou, "f")
                 structure_builder.set_anisou(anisou_array)
         # Now try to set the cell
         try:
-            a = float(mmcif_dict["_cell.length_a"])
-            b = float(mmcif_dict["_cell.length_b"])
-            c = float(mmcif_dict["_cell.length_c"])
-            alpha = float(mmcif_dict["_cell.angle_alpha"])
-            beta = float(mmcif_dict["_cell.angle_beta"])
-            gamma = float(mmcif_dict["_cell.angle_gamma"])
-            cell = numpy.array((a, b, c, alpha, beta, gamma), 'f')
-            spacegroup = mmcif_dict["_symmetry.space_group_name_H-M"]
+            a = float(mmcif_dict["_cell.length_a"][0])
+            b = float(mmcif_dict["_cell.length_b"][0])
+            c = float(mmcif_dict["_cell.length_c"][0])
+            alpha = float(mmcif_dict["_cell.angle_alpha"][0])
+            beta = float(mmcif_dict["_cell.angle_beta"][0])
+            gamma = float(mmcif_dict["_cell.angle_gamma"][0])
+            cell = numpy.array((a, b, c, alpha, beta, gamma), "f")
+            spacegroup = mmcif_dict["_symmetry.space_group_name_H-M"][0]
             spacegroup = spacegroup[1:-1]  # get rid of quotes!!
             if spacegroup is None:
                 raise Exception
             structure_builder.set_symmetry(spacegroup, cell)
         except Exception:
-            pass    # no cell found, so just ignore
+            pass  # no cell found, so just ignore
 
 
-class FastMMCIFParser(object):
+class FastMMCIFParser:
     """Parse an MMCIF file and return a Structure object."""
 
     def __init__(self, structure_builder=None, QUIET=False):
@@ -230,6 +293,7 @@ class FastMMCIFParser(object):
          - QUIET - Evaluated as a Boolean. If true, warnings issued in constructing
            the SMCRA data will be suppressed. If false (DEFAULT), they will be shown.
            These warnings might be indicative of problems in the mmCIF file!
+
         """
         if structure_builder is not None:
             self._structure_builder = structure_builder
@@ -248,6 +312,7 @@ class FastMMCIFParser(object):
         Arguments:
          - structure_id - string, the id that will be used for the structure
          - filename - name of the mmCIF file OR an open filehandle
+
         """
         with warnings.catch_warnings():
             if self.QUIET:
@@ -261,20 +326,25 @@ class FastMMCIFParser(object):
 
     def _build_structure(self, structure_id, filehandle):
 
+        # two special chars as placeholders in the mmCIF format
+        # for item values that cannot be explicitly assigned
+        # see: pdbx/mmcif syntax web page
+        _unassigned = {".", "?"}
+
         # Read only _atom_site. and atom_site_anisotrop entries
         read_atom, read_aniso = False, False
         _fields, _records = [], []
         _anisof, _anisors = [], []
         for line in filehandle:
-            if line.startswith('_atom_site.'):
+            if line.startswith("_atom_site."):
                 read_atom = True
                 _fields.append(line.strip())
-            elif line.startswith('_atom_site_anisotrop.'):
+            elif line.startswith("_atom_site_anisotrop."):
                 read_aniso = True
                 _anisof.append(line.strip())
-            elif read_atom and line.startswith('#'):
+            elif read_atom and line.startswith("#"):
                 read_atom = False
-            elif read_aniso and line.startswith('#'):
+            elif read_aniso and line.startswith("#"):
                 read_aniso = False
             elif read_atom:
                 _records.append(line.strip())
@@ -299,7 +369,6 @@ class FastMMCIFParser(object):
         except KeyError:
             element_list = None
 
-        seq_id_list = mmcif_dict["_atom_site.label_seq_id"]
         chain_id_list = mmcif_dict["_atom_site.auth_asym_id"]
 
         x_list = [float(x) for x in mmcif_dict["_atom_site.Cartn_x"]]
@@ -318,15 +387,15 @@ class FastMMCIFParser(object):
             serial_list = None
         except ValueError:
             # Invalid model number (malformed file)
-            raise PDBConstructionException("Invalid model number")
+            raise PDBConstructionException("Invalid model number") from None
 
         try:
-            aniso_u11 = mmcif_dict["_atom_site.aniso_U[1][1]"]
-            aniso_u12 = mmcif_dict["_atom_site.aniso_U[1][2]"]
-            aniso_u13 = mmcif_dict["_atom_site.aniso_U[1][3]"]
-            aniso_u22 = mmcif_dict["_atom_site.aniso_U[2][2]"]
-            aniso_u23 = mmcif_dict["_atom_site.aniso_U[2][3]"]
-            aniso_u33 = mmcif_dict["_atom_site.aniso_U[3][3]"]
+            aniso_u11 = mmcif_dict["_atom_site_anisotrop.U[1][1]"]
+            aniso_u12 = mmcif_dict["_atom_site_anisotrop.U[1][2]"]
+            aniso_u13 = mmcif_dict["_atom_site_anisotrop.U[1][3]"]
+            aniso_u22 = mmcif_dict["_atom_site_anisotrop.U[2][2]"]
+            aniso_u23 = mmcif_dict["_atom_site_anisotrop.U[2][3]"]
+            aniso_u33 = mmcif_dict["_atom_site_anisotrop.U[3][3]"]
             aniso_flag = 1
         except KeyError:
             # no anisotropic B factors
@@ -364,24 +433,25 @@ class FastMMCIFParser(object):
             resname = residue_id_list[i]
             chainid = chain_id_list[i]
             altloc = alt_list[i]
-            if altloc == ".":
+            if altloc in _unassigned:
                 altloc = " "
             int_resseq = int(seq_id_list[i])
             icode = icode_list[i]
-            if icode == "?":
+            if icode in _unassigned:
                 icode = " "
-            name = atom_id_list[i].strip('"')  # Remove occasional " from quoted atom names (e.g. xNA)
+            # Remove occasional " from quoted atom names (e.g. xNA)
+            name = atom_id_list[i].strip('"')
 
             # occupancy & B factor
             try:
                 tempfactor = float(b_factor_list[i])
             except ValueError:
-                raise PDBConstructionException("Invalid or missing B factor")
+                raise PDBConstructionException("Invalid or missing B factor") from None
 
             try:
                 occupancy = float(occupancy_list[i])
             except ValueError:
-                raise PDBConstructionException("Invalid or missing occupancy")
+                raise PDBConstructionException("Invalid or missing occupancy") from None
 
             fieldname = fieldname_list[i]
             if fieldname == "HETATM":
@@ -417,31 +487,20 @@ class FastMMCIFParser(object):
                 current_resname = resname
                 structure_builder.init_residue(resname, hetatm_flag, int_resseq, icode)
 
-            coord = numpy.array((x, y, z), 'f')
+            coord = numpy.array((x, y, z), "f")
             element = element_list[i] if element_list else None
-            structure_builder.init_atom(name, coord, tempfactor, occupancy, altloc,
-                name, element=element)
-            if aniso_flag == 1:
-                u = (aniso_u11[i], aniso_u12[i], aniso_u13[i],
-                    aniso_u22[i], aniso_u23[i], aniso_u33[i])
-                mapped_anisou = [float(x) for x in u]
-                anisou_array = numpy.array(mapped_anisou, 'f')
+            structure_builder.init_atom(
+                name, coord, tempfactor, occupancy, altloc, name, element=element
+            )
+            if aniso_flag == 1 and i < len(aniso_u11):
+                u = (
+                    aniso_u11[i],
+                    aniso_u12[i],
+                    aniso_u13[i],
+                    aniso_u22[i],
+                    aniso_u23[i],
+                    aniso_u33[i],
+                )
+                mapped_anisou = [float(_) for _ in u]
+                anisou_array = numpy.array(mapped_anisou, "f")
                 structure_builder.set_anisou(anisou_array)
-
-if __name__ == "__main__":
-    import sys
-
-    if len(sys.argv) != 2:
-        print("Usage: python MMCIFparser.py filename")
-        raise SystemExit
-    filename = sys.argv[1]
-
-    p = MMCIFParser()
-
-    structure = p.get_structure("test", filename)
-
-    for model in structure.get_list():
-        print(model)
-        for chain in model.get_list():
-            print(chain)
-            print("Found %d residues." % len(chain.get_list()))
