@@ -1,4 +1,10 @@
 #!/usr/bin/env python
+# Copyright 2003, by Kristian Rother. All rights reserved.
+#
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 #
 # PDBList.py
 #
@@ -23,16 +29,12 @@
 #   homepage : http://jaceksmietanski.net
 #   email    : jacek.smietanski@ii.uj.edu.pl
 #
-# This code is released under the conditions of the Biopython license.
-# Please see the LICENSE file that should have been included as part of this
-# package.
 # It may be distributed freely with respect to the original authors.
 # Any maintainer of the Biopython code may change this notice
 # when appropriate.
 
 """Access the PDB over the internet (e.g. to download structures)."""
 
-from __future__ import print_function
 
 import contextlib
 import gzip
@@ -41,14 +43,12 @@ import shutil
 import re
 import sys
 
-# Importing these functions with leading underscore as not intended for reuse
-from Bio._py3k import _as_string
-from Bio._py3k import urlopen as _urlopen
-from Bio._py3k import urlretrieve as _urlretrieve
-from Bio._py3k import urlcleanup as _urlcleanup
+from urllib.request import urlopen
+from urllib.request import urlretrieve
+from urllib.request import urlcleanup
 
 
-class PDBList(object):
+class PDBList:
     """Quick access to the structure lists on the PDB or its mirrors.
 
     This class provides quick access to the structure lists on the
@@ -83,17 +83,28 @@ class PDBList(object):
     http://www.pdb.org/.
     """
 
-    def __init__(self, server='ftp://ftp.wwpdb.org', pdb=os.getcwd(),
-                 obsolete_pdb=None):
-        """Initialize the class with the default server or a custom one."""
+    def __init__(
+        self, server="ftp://ftp.wwpdb.org", pdb=None, obsolete_pdb=None, verbose=True
+    ):
+        """Initialize the class with the default server or a custom one.
+
+        Argument pdb is the local path to use, defaulting to the current
+        directory at the moment of initialisation.
+        """
         self.pdb_server = server  # remote pdb server
-        self.local_pdb = pdb  # local pdb file tree
+        if pdb:
+            self.local_pdb = pdb  # local pdb file tree
+        else:
+            self.local_pdb = os.getcwd()
+
+        # enable or disable verbose
+        self._verbose = verbose
 
         # local file tree for obsolete pdb files
         if obsolete_pdb:
             self.obsolete_pdb = obsolete_pdb
         else:
-            self.obsolete_pdb = os.path.join(self.local_pdb, 'obsolete')
+            self.obsolete_pdb = os.path.join(self.local_pdb, "obsolete")
             if not os.access(self.obsolete_pdb, os.F_OK):
                 os.makedirs(self.obsolete_pdb)
 
@@ -102,30 +113,35 @@ class PDBList(object):
 
     @staticmethod
     def _print_default_format_warning(file_format):
-        """Temporary warning (similar to a deprecation warning) that files are being downloaded in mmCIF"""
+        """Print a warning to stdout (PRIVATE).
+
+        Temporary warning (similar to a deprecation warning) that files
+        are being downloaded in mmCIF.
+        """
         if file_format is None:
-            sys.stderr.write("WARNING: The default download format has changed from PDB to PDBx/mmCif\n")
+            sys.stderr.write(
+                "WARNING: The default download format has changed from PDB to PDBx/mmCif\n"
+            )
             return "mmCif"
         return file_format
 
     @staticmethod
     def get_status_list(url):
-        """Retrieves a list of pdb codes in the weekly pdb status file
-        from the given URL. Used by get_recent_files.
+        """Retrieve a list of pdb codes in the weekly pdb status file from given URL.
 
-        Typical contents of the list files parsed by this method is now
-        very simply one PDB name per line.
+        Used by get_recent_changes. Typical contents of the list files parsed
+        by this method is now very simply - one PDB name per line.
         """
-        with contextlib.closing(_urlopen(url)) as handle:
+        with contextlib.closing(urlopen(url)) as handle:
             answer = []
             for line in handle:
                 pdb = line.strip()
                 assert len(pdb) == 4
-                answer.append(_as_string(pdb))
+                answer.append(pdb.decode())
         return answer
 
     def get_recent_changes(self):
-        """Returns three lists of the newest weekly files (added,mod,obsolete).
+        """Return three lists of the newest weekly files (added,mod,obsolete).
 
         Reads the directories with changed entries from the PDB server and
         returns a tuple of three URL's to the files of new, modified and
@@ -140,28 +156,30 @@ class PDBList(object):
             -rw-r--r--   1 1002     sysadmin    1327 Mar 12  2001 README
 
         """
-        path = self.pdb_server + '/pub/pdb/data/status/latest/'
+        path = self.pdb_server + "/pub/pdb/data/status/latest/"
 
         # Retrieve the lists
-        added = self.get_status_list(path + 'added.pdb')
-        modified = self.get_status_list(path + 'modified.pdb')
-        obsolete = self.get_status_list(path + 'obsolete.pdb')
+        added = self.get_status_list(path + "added.pdb")
+        modified = self.get_status_list(path + "modified.pdb")
+        obsolete = self.get_status_list(path + "obsolete.pdb")
         return [added, modified, obsolete]
 
     def get_all_entries(self):
-        """Retrieves a big file containing all the PDB entries and some annotation.
+        """Retrieve the big file containing all the PDB entries and some annotation.
 
         Returns a list of PDB codes in the index file.
         """
-        url = self.pdb_server + '/pub/pdb/derived_data/index/entries.idx'
-        print("Retrieving index file. Takes about 27 MB.")
-        with contextlib.closing(_urlopen(url)) as handle:
-            all_entries = [_as_string(line[:4]) for line in handle.readlines()[2:]
-                           if len(line) > 4]
+        url = self.pdb_server + "/pub/pdb/derived_data/index/entries.idx"
+        if self._verbose:
+            print("Retrieving index file. Takes about 27 MB.")
+        with contextlib.closing(urlopen(url)) as handle:
+            all_entries = [
+                line[:4].decode() for line in handle.readlines()[2:] if len(line) > 4
+            ]
         return all_entries
 
     def get_all_obsolete(self):
-        """Returns a list of all obsolete entries ever in the PDB.
+        """Return a list of all obsolete entries ever in the PDB.
 
         Returns a list of all obsolete pdb codes that have ever been
         in the PDB.
@@ -183,8 +201,8 @@ class PDBList(object):
             ...
 
         """
-        url = self.pdb_server + '/pub/pdb/data/status/obsolete.dat'
-        with contextlib.closing(_urlopen(url)) as handle:
+        url = self.pdb_server + "/pub/pdb/data/status/obsolete.dat"
+        with contextlib.closing(urlopen(url)) as handle:
             # Extract pdb codes. Could use a list comprehension, but I want
             # to include an assert to check for mis-reading the data.
             obsolete = []
@@ -193,10 +211,12 @@ class PDBList(object):
                     continue
                 pdb = line.split()[2]
                 assert len(pdb) == 4
-                obsolete.append(_as_string(pdb))
+                obsolete.append(pdb.decode())
         return obsolete
 
-    def retrieve_pdb_file(self, pdb_code, obsolete=False, pdir=None, file_format=None, overwrite=False):
+    def retrieve_pdb_file(
+        self, pdb_code, obsolete=False, pdir=None, file_format=None, overwrite=False
+    ):
         """Fetch PDB structure file from PDB server, and store it locally.
 
         The PDB structure's file name is returned as a single string.
@@ -204,10 +224,10 @@ class PDBList(object):
 
         NOTE. The default download format has changed from PDB to PDBx/mmCif
 
-        @param pdb_code: 4-symbols structure Id from PDB (e.g. 3J92).
-        @type pdb_code: string
+        :param pdb_code: 4-symbols structure Id from PDB (e.g. 3J92).
+        :type pdb_code: string
 
-        @param file_format:
+        :param file_format:
             File format. Available options:
 
             * "mmCif" (default, PDBx/mmCif file),
@@ -216,12 +236,12 @@ class PDBList(object):
             * "mmtf" (highly compressed),
             * "bundle" (PDB formatted archive for large structure}
 
-        @type file_format: string
+        :type file_format: string
 
-        @param overwrite: if set to True, existing structure files will be overwritten. Default: False
-        @type overwrite: bool
+        :param overwrite: if set to True, existing structure files will be overwritten. Default: False
+        :type overwrite: bool
 
-        @param obsolete:
+        :param obsolete:
             Has a meaning only for obsolete structures. If True, download the obsolete structure
             to 'obsolete' folder, otherwise download won't be performed.
             This option doesn't work for mmtf format as obsoleted structures aren't stored in mmtf.
@@ -231,34 +251,58 @@ class PDBList(object):
             and you face the "structure doesn't exists" error.
             Default: False
 
-        @type obsolete: bool
+        :type obsolete: bool
 
-        @param pdir: put the file in this directory (default: create a PDB-style directory tree)
-        @type pdir: string
+        :param pdir: put the file in this directory (default: create a PDB-style directory tree)
+        :type pdir: string
 
-        @return: filename
-        @rtype: string
+        :return: filename
+        :rtype: string
         """
-        file_format = self._print_default_format_warning(file_format)  # Deprecation warning
+        # Deprecation warning
+        file_format = self._print_default_format_warning(file_format)
 
         # Get the compressed PDB structure
         code = pdb_code.lower()
-        archive = {'pdb': 'pdb%s.ent.gz', 'mmCif': '%s.cif.gz', 'xml': '%s.xml.gz', 'mmtf': '%s',
-                   'bundle': '%s-pdb-bundle.tar.gz'}
+        archive = {
+            "pdb": "pdb%s.ent.gz",
+            "mmCif": "%s.cif.gz",
+            "xml": "%s.xml.gz",
+            "mmtf": "%s",
+            "bundle": "%s-pdb-bundle.tar.gz",
+        }
         archive_fn = archive[file_format] % code
 
         if file_format not in archive.keys():
-            raise("Specified file_format %s doesn't exists or is not supported. Maybe a typo. "
-                  "Please, use one of the following: mmCif, pdb, xml, mmtf, bundle" % file_format)
+            raise (
+                "Specified file_format %s doesn't exists or is not supported. Maybe a "
+                "typo. Please, use one of the following: mmCif, pdb, xml, mmtf, bundle"
+                % file_format
+            )
 
-        if file_format in ('pdb', 'mmCif', 'xml'):
+        if file_format in ("pdb", "mmCif", "xml"):
             pdb_dir = "divided" if not obsolete else "obsolete"
-            file_type = "pdb" if file_format == "pdb" else "mmCIF" if file_format == "mmCif" else "XML"
-            url = (self.pdb_server + '/pub/pdb/data/structures/%s/%s/%s/%s' % (pdb_dir, file_type, code[1:3], archive_fn))
-        elif file_format == 'bundle':
-            url = (self.pdb_server + '/pub/pdb/compatible/pdb_bundle/%s/%s/%s' % (code[1:3], code, archive_fn))
+            file_type = (
+                "pdb"
+                if file_format == "pdb"
+                else "mmCIF"
+                if file_format == "mmCif"
+                else "XML"
+            )
+            url = self.pdb_server + "/pub/pdb/data/structures/%s/%s/%s/%s" % (
+                pdb_dir,
+                file_type,
+                code[1:3],
+                archive_fn,
+            )
+        elif file_format == "bundle":
+            url = self.pdb_server + "/pub/pdb/compatible/pdb_bundle/%s/%s/%s" % (
+                code[1:3],
+                code,
+                archive_fn,
+            )
         else:
-            url = ('http://mmtf.rcsb.org/v1.0/full/%s' % code)
+            url = "http://mmtf.rcsb.org/v1.0/full/%s" % code
 
         # Where does the final PDB file get saved?
         if pdir is None:
@@ -270,26 +314,33 @@ class PDBList(object):
         if not os.access(path, os.F_OK):
             os.makedirs(path)
         filename = os.path.join(path, archive_fn)
-        final = {'pdb': 'pdb%s.ent', 'mmCif': '%s.cif', 'xml': '%s.xml',
-                 'mmtf': '%s.mmtf', 'bundle': '%s-pdb-bundle.tar'}
+        final = {
+            "pdb": "pdb%s.ent",
+            "mmCif": "%s.cif",
+            "xml": "%s.xml",
+            "mmtf": "%s.mmtf",
+            "bundle": "%s-pdb-bundle.tar",
+        }
         final_file = os.path.join(path, final[file_format] % code)
 
         # Skip download if the file already exists
         if not overwrite:
             if os.path.exists(final_file):
-                print("Structure exists: '%s' " % final_file)
+                if self._verbose:
+                    print("Structure exists: '%s' " % final_file)
                 return final_file
 
         # Retrieve the file
-        print("Downloading PDB structure '%s'..." % pdb_code)
+        if self._verbose:
+            print("Downloading PDB structure '%s'..." % pdb_code)
         try:
-            _urlcleanup()
-            _urlretrieve(url, filename)
-        except IOError:
+            urlcleanup()
+            urlretrieve(url, filename)
+        except OSError:
             print("Desired structure doesn't exists")
         else:
-            with gzip.open(filename, 'rb') as gz:
-                with open(final_file, 'wb') as out:
+            with gzip.open(filename, "rb") as gz:
+                with open(final_file, "wb") as out:
                     out.writelines(gz)
             os.remove(filename)
         return final_file
@@ -305,7 +356,8 @@ class PDBList(object):
         assert os.path.isdir(self.local_pdb)
         assert os.path.isdir(self.obsolete_pdb)
 
-        file_format = self._print_default_format_warning(file_format)  # Deprecation warning
+        # Deprecation warning
+        file_format = self._print_default_format_warning(file_format)
 
         new, modified, obsolete = self.get_recent_changes()
 
@@ -313,21 +365,21 @@ class PDBList(object):
             try:
                 self.retrieve_pdb_file(pdb_code, file_format=file_format)
             except Exception:
-                print('error %s\n' % pdb_code)
+                print("error %s\n" % pdb_code)
                 # you can insert here some more log notes that
                 # something has gone wrong.
 
         # Move the obsolete files to a special folder
         for pdb_code in obsolete:
             if self.flat_tree:
-                old_file = os.path.join(self.local_pdb,
-                                        'pdb%s.ent' % pdb_code)
+                old_file = os.path.join(self.local_pdb, "pdb%s.ent" % pdb_code)
                 new_dir = self.obsolete_pdb
             else:
-                old_file = os.path.join(self.local_pdb, pdb_code[1:3],
-                                        'pdb%s.ent' % pdb_code)
+                old_file = os.path.join(
+                    self.local_pdb, pdb_code[1:3], "pdb%s.ent" % pdb_code
+                )
                 new_dir = os.path.join(self.obsolete_pdb, pdb_code[1:3])
-            new_file = os.path.join(new_dir, 'pdb%s.ent' % pdb_code)
+            new_file = os.path.join(new_dir, "pdb%s.ent" % pdb_code)
             if os.path.isfile(old_file):
                 if not os.path.isdir(new_dir):
                     os.mkdir(new_dir)
@@ -336,20 +388,24 @@ class PDBList(object):
                 except Exception:
                     print("Could not move %s to obsolete folder" % old_file)
             elif os.path.isfile(new_file):
-                print("Obsolete file %s already moved" % old_file)
+                if self._verbose:
+                    print("Obsolete file %s already moved" % old_file)
             else:
-                print("Obsolete file %s is missing" % old_file)
+                if self._verbose:
+                    print("Obsolete file %s is missing" % old_file)
 
-    def download_pdb_files(self, pdb_codes, obsolete=False, pdir=None, file_format=None, overwrite=False):
+    def download_pdb_files(
+        self, pdb_codes, obsolete=False, pdir=None, file_format=None, overwrite=False
+    ):
         """Fetch set of PDB structure files from the PDB server and stores them locally.
 
         The PDB structure's file name is returned as a single string.
         If obsolete ``==`` True, the files will be saved in a special file tree.
 
-        @param pdb_codes: a list of 4-symbols structure Ids from PDB
-        @type pdb_codes: list of strings
+        :param pdb_codes: a list of 4-symbols structure Ids from PDB
+        :type pdb_codes: list of strings
 
-        @param file_format:
+        :param file_format:
             File format. Available options:
 
             * "mmCif" (default, PDBx/mmCif file),
@@ -358,34 +414,41 @@ class PDBList(object):
             * "mmtf" (highly compressed),
             * "bundle" (PDB formatted archive for large structure}
 
-        @param overwrite: if set to True, existing structure files will be overwritten. Default: False
-        @type overwrite: bool
+        :param overwrite: if set to True, existing structure files will be overwritten. Default: False
+        :type overwrite: bool
 
-        @param obsolete:
+        :param obsolete:
             Has a meaning only for obsolete structures.
             If True, download the obsolete structure
             to 'obsolete' folder, otherwise download won't be performed.
             This option doesn't work for mmtf format as obsoleted structures are not availbe as mmtf.
             (default: False)
 
-        @type obsolete: bool
+        :type obsolete: bool
 
-        @param pdir: put the file in this directory (default: create a PDB-style directory tree)
-        @type pdir: string
+        :param pdir: put the file in this directory (default: create a PDB-style directory tree)
+        :type pdir: string
 
-        @return: filenames
-        @rtype: string
+        :return: filenames
+        :rtype: string
         """
-        file_format = self._print_default_format_warning(file_format)  # Deprecation warning
+        # Deprecation warning
+        file_format = self._print_default_format_warning(file_format)
         for pdb_code in pdb_codes:
-            self.retrieve_pdb_file(pdb_code, obsolete=obsolete, pdir=pdir, file_format=file_format, overwrite=overwrite)
+            self.retrieve_pdb_file(
+                pdb_code,
+                obsolete=obsolete,
+                pdir=pdir,
+                file_format=file_format,
+                overwrite=overwrite,
+            )
 
     def download_entire_pdb(self, listfile=None, file_format=None):
         """Retrieve all PDB entries not present in the local PDB copy.
 
-        @param listfile: filename to which all PDB codes will be written (optional)
+        :param listfile: filename to which all PDB codes will be written (optional)
 
-        @param file_format:
+        :param file_format:
             File format. Available options:
 
             * "mmCif" (default, PDBx/mmCif file),
@@ -396,46 +459,48 @@ class PDBList(object):
 
         NOTE. The default download format has changed from PDB to PDBx/mmCif
         """
-        file_format = self._print_default_format_warning(file_format)  # Deprecation warning
+        # Deprecation warning
+        file_format = self._print_default_format_warning(file_format)
         entries = self.get_all_entries()
         for pdb_code in entries:
             self.retrieve_pdb_file(pdb_code, file_format=file_format)
         # Write the list
         if listfile:
-            with open(listfile, 'w') as outfile:
-                outfile.writelines((x + '\n' for x in entries))
+            with open(listfile, "w") as outfile:
+                outfile.writelines(x + "\n" for x in entries)
 
     def download_obsolete_entries(self, listfile=None, file_format=None):
-        """Retrieve all obsolete PDB entries not present in the local obsolete
-        PDB copy.
+        """Retrieve all obsolete PDB entries not present in local obsolete PDB copy.
 
-        @param listfile: filename to which all PDB codes will be written (optional)
+        :param listfile: filename to which all PDB codes will be written (optional)
 
-        @param file_format: file format. Available options:
+        :param file_format: file format. Available options:
             "mmCif" (default, PDBx/mmCif file),
             "pdb" (format PDB),
             "xml" (PMDML/XML format),
 
         NOTE. The default download format has changed from PDB to PDBx/mmCif
         """
-        file_format = self._print_default_format_warning(file_format)  # Deprecation warning
+        # Deprecation warning
+        file_format = self._print_default_format_warning(file_format)
         entries = self.get_all_obsolete()
         for pdb_code in entries:
             self.retrieve_pdb_file(pdb_code, obsolete=True, file_format=file_format)
 
         # Write the list
         if listfile:
-            with open(listfile, 'w') as outfile:
-                outfile.writelines((x + '\n' for x in entries))
+            with open(listfile, "w") as outfile:
+                outfile.writelines(x + "\n" for x in entries)
 
-    def get_seqres_file(self, savefile='pdb_seqres.txt'):
-        """Retrieves and save a (big) file containing all the sequences of PDB entries."""
-        print("Retrieving sequence file (takes over 110 MB).")
-        url = self.pdb_server + '/pub/pdb/derived_data/pdb_seqres.txt'
-        _urlretrieve(url, savefile)
+    def get_seqres_file(self, savefile="pdb_seqres.txt"):
+        """Retrieve and save a (big) file containing all the sequences of PDB entries."""
+        if self._verbose:
+            print("Retrieving sequence file (takes over 110 MB).")
+        url = self.pdb_server + "/pub/pdb/derived_data/pdb_seqres.txt"
+        urlretrieve(url, savefile)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
 
     doc = """PDBList.py
     (c) Kristian Rother 2003, Wiktoria Karwicka & Jacek Smietanski 2016
@@ -474,11 +539,11 @@ if __name__ == '__main__':
         pl = PDBList(pdb=pdb_path)
         if len(sys.argv) > 3:
             for option in sys.argv[3:]:
-                if option == '-d':
+                if option == "-d":
                     pl.flat_tree = True
-                elif option == '-o':
+                elif option == "-o":
                     overwrite = True
-                elif option in ('-pdb', '-xml', '-mmtf'):
+                elif option in ("-pdb", "-xml", "-mmtf"):
                     file_format = option[1:]
     else:
         pdb_path = os.getcwd()
@@ -486,25 +551,29 @@ if __name__ == '__main__':
         pl.flat_tree = True
 
     if len(sys.argv) > 1:
-        if sys.argv[1] == 'update':
+        if sys.argv[1] == "update":
             # update PDB
             print("updating local PDB at " + pdb_path)
             pl.update_pdb(file_format=file_format)
 
-        elif sys.argv[1] == 'all':
+        elif sys.argv[1] == "all":
             # get the entire PDB
             pl.download_entire_pdb(file_format=file_format)
 
-        elif sys.argv[1] == 'obsol':
+        elif sys.argv[1] == "obsol":
             # get all obsolete entries
             pl.download_obsolete_entries(pdb_path, file_format=file_format)
 
         elif len(sys.argv[1]) == 4 and sys.argv[1][0].isdigit():
             # get single PDB entry
-            pl.retrieve_pdb_file(sys.argv[1], pdir=pdb_path, file_format=file_format, overwrite=overwrite)
+            pl.retrieve_pdb_file(
+                sys.argv[1], pdir=pdb_path, file_format=file_format, overwrite=overwrite
+            )
 
-        elif sys.argv[1][0] == '(':
+        elif sys.argv[1][0] == "(":
             # get a set of PDB entries
             pdb_ids = re.findall(sys.argv[1], "[0-9A-Za-z]{4}")
             for pdb_id in pdb_ids:
-                pl.retrieve_pdb_file(pdb_id, pdir=pdb_path, file_format=file_format, overwrite=overwrite)
+                pl.retrieve_pdb_file(
+                    pdb_id, pdir=pdb_path, file_format=file_format, overwrite=overwrite
+                )
