@@ -5,9 +5,6 @@
 # package.
 """Bio.SearchIO objects to model high scoring regions between query and hit."""
 
-from __future__ import print_function
-from Bio._py3k import basestring
-
 import warnings
 from operator import ge, le
 
@@ -17,8 +14,13 @@ from Bio.Alphabet import single_letter_alphabet
 from Bio.Seq import Seq
 from Bio.SeqRecord import SeqRecord
 
-from Bio._utils import getattr_str, trim_str
-from Bio.SearchIO._utils import singleitem, allitems, fullcascade, fragcascade
+from Bio.SearchIO._utils import (
+    singleitem,
+    allitems,
+    fullcascade,
+    fragcascade,
+    getattr_str,
+)
 
 from ._base import _BaseHSP
 
@@ -312,14 +314,10 @@ class HSP(_BaseHSP):
         """Return number of HSPs items."""
         return len(self._items)
 
-    # Python 3:
     def __bool__(self):
         """Return True if it has HSPs."""
         return bool(self._items)
 
-    # Python 2:
-    __nonzero__ = __bool__
-
     def __str__(self):
         """Return a human readable summary of the HSP object."""
         lines = []
@@ -353,12 +351,14 @@ class HSP(_BaseHSP):
                 query_end = getattr_str(block, "query_end")
                 query_range = "[%s:%s]" % (query_start, query_end)
                 # max column length is 20
-                query_range = trim_str(query_range, 22, "~]")
+                query_range = (
+                    query_range[:20] + "~]" if len(query_range) > 22 else query_range
+                )
                 # hit region
                 hit_start = getattr_str(block, "hit_start")
                 hit_end = getattr_str(block, "hit_end")
                 hit_range = "[%s:%s]" % (hit_start, hit_end)
-                hit_range = trim_str(hit_range, 22, "~]")
+                hit_range = hit_range[:20] + "~]" if len(hit_range) > 22 else hit_range
                 # append the hsp row
                 lines.append(pattern % (str(idx), aln_span, query_range, hit_range))
 
@@ -846,7 +846,7 @@ class HSPFragment(_BaseHSP):
             # similarity line
             simil = ""
             if "similarity" in self.aln_annotation and isinstance(
-                self.aln_annotation.get("similarity"), basestring
+                self.aln_annotation.get("similarity"), str
             ):
                 simil = self.aln_annotation["similarity"]
 
@@ -883,7 +883,7 @@ class HSPFragment(_BaseHSP):
         if seq is None:
             return seq  # return immediately if seq is None
         else:
-            if not isinstance(seq, (basestring, SeqRecord)):
+            if not isinstance(seq, (str, SeqRecord)):
                 raise TypeError(
                     "%s sequence must be a string or a SeqRecord object." % seq_type
                 )
@@ -908,7 +908,7 @@ class HSPFragment(_BaseHSP):
             seq.name = seq_name
             seq.features = seq_feats
             seq.seq.alphabet = self.alphabet
-        elif isinstance(seq, basestring):
+        elif isinstance(seq, str):
             seq = SeqRecord(
                 Seq(seq, self.alphabet),
                 id=seq_id,
@@ -983,7 +983,9 @@ class HSPFragment(_BaseHSP):
         # length of alignment (gaps included)
         # alignment span can be its own attribute, or computed from
         # query / hit length
-        if not hasattr(self, "_aln_span"):
+        try:
+            self._aln_span
+        except AttributeError:
             if self.query is not None:
                 self._aln_span = len(self.query)
             elif self.hit is not None:
