@@ -26,12 +26,10 @@ keys and offsets in an SQLite database - which can be re-used to avoid
 re-indexing the file for use another time.
 """
 
-from __future__ import print_function
 
 import re
 from io import BytesIO
-from Bio._py3k import StringIO
-from Bio._py3k import _bytes_to_string
+from io import StringIO
 
 from Bio import SeqIO
 from Bio import Alphabet
@@ -52,9 +50,11 @@ class SeqFileRandomAccess(_IndexedSeqFileProxy):
         # The following alphabet code is a bit nasty... duplicates logic in
         # Bio.SeqIO.parse()
         if alphabet is None:
+
             def _parse(handle):
                 """Dynamically generated parser function (PRIVATE)."""
                 return next(i(handle))
+
         else:
             # TODO - Detect alphabet support ONCE at __init__
             def _parse(handle):
@@ -63,12 +63,13 @@ class SeqFileRandomAccess(_IndexedSeqFileProxy):
                     return next(i(handle, alphabet=alphabet))
                 except TypeError:
                     return next(SeqIO._force_alphabet(i(handle), alphabet))
+
         self._parse = _parse
 
     def get(self, offset):
         """Return SeqRecord."""
         # Should be overridden for binary file formats etc:
-        return self._parse(StringIO(_bytes_to_string(self.get_raw(offset))))
+        return self._parse(StringIO(self.get_raw(offset).decode()))
 
 
 ####################
@@ -84,9 +85,15 @@ class SffRandomAccess(SeqFileRandomAccess):
     def __init__(self, filename, format, alphabet):
         """Initialize the class."""
         SeqFileRandomAccess.__init__(self, filename, format, alphabet)
-        header_length, index_offset, index_length, number_of_reads, \
-            self._flows_per_read, self._flow_chars, self._key_sequence \
-            = SeqIO.SffIO._sff_file_header(self._handle)
+        (
+            header_length,
+            index_offset,
+            index_length,
+            number_of_reads,
+            self._flows_per_read,
+            self._flow_chars,
+            self._key_sequence,
+        ) = SeqIO.SffIO._sff_file_header(self._handle)
 
     def __iter__(self):
         """Load any index block in the file, or build it the slow way (PRIVATE)."""
@@ -95,9 +102,15 @@ class SffRandomAccess(SeqFileRandomAccess):
         handle = self._handle
         handle.seek(0)
         # Alread did this in __init__ but need handle in right place
-        header_length, index_offset, index_length, number_of_reads, \
-            self._flows_per_read, self._flow_chars, self._key_sequence \
-            = SeqIO.SffIO._sff_file_header(handle)
+        (
+            header_length,
+            index_offset,
+            index_length,
+            number_of_reads,
+            self._flows_per_read,
+            self._flow_chars,
+            self._key_sequence,
+        ) = SeqIO.SffIO._sff_file_header(handle)
         if index_offset and index_length:
             # There is an index provided, try this the fast way:
             count = 0
@@ -108,14 +121,17 @@ class SffRandomAccess(SeqFileRandomAccess):
                     yield name, offset, 0
                     count += 1
                 if count != number_of_reads:
-                    raise ValueError("Indexed %i records, expected %i"
-                                     % (count, number_of_reads))
+                    raise ValueError(
+                        "Indexed %i records, expected %i" % (count, number_of_reads)
+                    )
                 # If that worked, call _check_eof ...
             except ValueError as err:
                 import warnings
                 from Bio import BiopythonParserWarning
-                warnings.warn("Could not parse the SFF index: %s" % err,
-                              BiopythonParserWarning)
+
+                warnings.warn(
+                    "Could not parse the SFF index: %s" % err, BiopythonParserWarning
+                )
                 assert count == 0, "Partially populated index"
                 handle.seek(0)
                 # Drop out to the slow way...
@@ -137,19 +153,22 @@ class SffRandomAccess(SeqFileRandomAccess):
             yield name, offset, 0
             count += 1
         if count != number_of_reads:
-            raise ValueError("Indexed %i records, expected %i"
-                             % (count, number_of_reads))
+            raise ValueError(
+                "Indexed %i records, expected %i" % (count, number_of_reads)
+            )
         SeqIO.SffIO._check_eof(handle, index_offset, index_length)
 
     def get(self, offset):
         """Return the SeqRecord starting at the given offset."""
         handle = self._handle
         handle.seek(offset)
-        return SeqIO.SffIO._sff_read_seq_record(handle,
-                                                self._flows_per_read,
-                                                self._flow_chars,
-                                                self._key_sequence,
-                                                self._alphabet)
+        return SeqIO.SffIO._sff_read_seq_record(
+            handle,
+            self._flows_per_read,
+            self._flow_chars,
+            self._key_sequence,
+            self._alphabet,
+        )
 
     def get_raw(self, offset):
         """Return the raw record from the file as a bytes string."""
@@ -165,36 +184,40 @@ class SffTrimedRandomAccess(SffRandomAccess):
         """Return the SeqRecord starting at the given offset."""
         handle = self._handle
         handle.seek(offset)
-        return SeqIO.SffIO._sff_read_seq_record(handle,
-                                                self._flows_per_read,
-                                                self._flow_chars,
-                                                self._key_sequence,
-                                                self._alphabet,
-                                                trim=True)
+        return SeqIO.SffIO._sff_read_seq_record(
+            handle,
+            self._flows_per_read,
+            self._flow_chars,
+            self._key_sequence,
+            self._alphabet,
+            trim=True,
+        )
 
 
 ###################
 # Simple indexers #
 ###################
 
+
 class SequentialSeqFileRandomAccess(SeqFileRandomAccess):
     """Random access to a simple sequential sequence file."""
 
     def __init__(self, filename, format, alphabet):
         """Initialize the class."""
         SeqFileRandomAccess.__init__(self, filename, format, alphabet)
-        marker = {"ace": b"CO ",
-                  "embl": b"ID ",
-                  "fasta": b">",
-                  "genbank": b"LOCUS ",
-                  "gb": b"LOCUS ",
-                  "imgt": b"ID ",
-                  "phd": b"BEGIN_SEQUENCE",
-                  "pir": b">..;",
-                  "qual": b">",
-                  "swiss": b"ID ",
-                  "uniprot-xml": b"<entry ",
-                  }[format]
+        marker = {
+            "ace": b"CO ",
+            "embl": b"ID ",
+            "fasta": b">",
+            "genbank": b"LOCUS ",
+            "gb": b"LOCUS ",
+            "imgt": b"ID ",
+            "phd": b"BEGIN_SEQUENCE",
+            "pir": b">..;",
+            "qual": b">",
+            "swiss": b"ID ",
+            "uniprot-xml": b"<entry ",
+        }[format]
         self._marker = marker
         self._marker_re = re.compile(b"^" + marker)
 
@@ -220,7 +243,7 @@ class SequentialSeqFileRandomAccess(SeqFileRandomAccess):
                 end_offset = handle.tell()
                 line = handle.readline()
                 if marker_re.match(line) or not line:
-                    yield _bytes_to_string(id), start_offset, length
+                    yield id.decode(), start_offset, length
                     start_offset = end_offset
                     break
                 else:
@@ -248,6 +271,7 @@ class SequentialSeqFileRandomAccess(SeqFileRandomAccess):
 # Fiddly indexers: GenBank, EMBL, ... #
 #######################################
 
+
 class GenBankRandomAccess(SequentialSeqFileRandomAccess):
     """Indexed dictionary like access to a GenBank file."""
 
@@ -281,8 +305,10 @@ class GenBankRandomAccess(SequentialSeqFileRandomAccess):
                 line = handle.readline()
                 if marker_re.match(line) or not line:
                     if not key:
-                        raise ValueError("Did not find usable ACCESSION/VERSION/LOCUS lines")
-                    yield _bytes_to_string(key), start_offset, length
+                        raise ValueError(
+                            "Did not find usable ACCESSION/VERSION/LOCUS lines"
+                        )
+                    yield key.decode(), start_offset, length
                     start_offset = end_offset
                     break
                 elif line.startswith(accession_marker):
@@ -294,7 +320,10 @@ class GenBankRandomAccess(SequentialSeqFileRandomAccess):
                 elif line.startswith(version_marker):
                     try:
                         version_id = line.rstrip().split()[1]
-                        if version_id.count(b".") == 1 and version_id.split(b".")[1].isdigit():
+                        if (
+                            version_id.count(b".") == 1
+                            and version_id.split(b".")[1].isdigit()
+                        ):
                             # This should mimic the GenBank parser...
                             key = version_id
                     except IndexError:
@@ -334,8 +363,7 @@ class EmblRandomAccess(SequentialSeqFileRandomAccess):
                 parts = line[3:].rstrip().split(b";")
                 if parts[1].strip().startswith(sv_marker):
                     # The SV bit gives the version
-                    key = parts[0].strip() + b"." + \
-                        parts[1].strip().split()[1]
+                    key = parts[0].strip() + b"." + parts[1].strip().split()[1]
                     setbysv = True
                 else:
                     key = parts[0].strip()
@@ -346,14 +374,13 @@ class EmblRandomAccess(SequentialSeqFileRandomAccess):
                 if key.endswith(b";"):
                     key = key[:-1]
             else:
-                raise ValueError(
-                    'Did not recognise the ID line layout:\n%r' % line)
+                raise ValueError("Did not recognise the ID line layout:\n%r" % line)
             while True:
                 end_offset = handle.tell()
                 line = handle.readline()
                 if marker_re.match(line) or not line:
                     end_offset = handle.tell() - len(line)
-                    yield _bytes_to_string(key), start_offset, length
+                    yield key.decode(), start_offset, length
                     start_offset = end_offset
                     break
                 elif line.startswith(ac_marker) and not setbysv:
@@ -394,7 +421,7 @@ class SwissRandomAccess(SequentialSeqFileRandomAccess):
                 end_offset = handle.tell()
                 line = handle.readline()
                 if marker_re.match(line) or not line:
-                    yield _bytes_to_string(key), start_offset, length
+                    yield key.decode(), start_offset, length
                     start_offset = end_offset
                     break
                 length += len(line)
@@ -429,13 +456,13 @@ class UniprotRandomAccess(SequentialSeqFileRandomAccess):
                 line = handle.readline()
                 if key is None and start_acc_marker in line:
                     assert end_acc_marker in line, line
-                    key = line[line.find(
-                        start_acc_marker) + 11:].split(b"<", 1)[0]
+                    key = line[line.find(start_acc_marker) + 11 :].split(b"<", 1)[0]
                     length += len(line)
                 elif end_entry_marker in line:
                     length += line.find(end_entry_marker) + 8
-                    end_offset = handle.tell() - len(line) \
-                        + line.find(end_entry_marker) + 8
+                    end_offset = (
+                        handle.tell() - len(line) + line.find(end_entry_marker) + 8
+                    )
                     assert start_offset + length == end_offset
                     break
                 elif marker_re.match(line) or not line:
@@ -444,9 +471,11 @@ class UniprotRandomAccess(SequentialSeqFileRandomAccess):
                 else:
                     length += len(line)
             if not key:
-                raise ValueError("Did not find <accession> line in bytes %i to %i"
-                                 % (start_offset, start_offset + length))
-            yield _bytes_to_string(key), start_offset, length
+                raise ValueError(
+                    "Did not find <accession> line in bytes %i to %i"
+                    % (start_offset, start_offset + length)
+                )
+            yield key.decode(), start_offset, length
             # Find start of next record
             while not marker_re.match(line) and line:
                 start_offset = handle.tell()
@@ -464,7 +493,7 @@ class UniprotRandomAccess(SequentialSeqFileRandomAccess):
             line = handle.readline()
             i = line.find(end_entry_marker)
             if i != -1:
-                data.append(line[:i + 8])
+                data.append(line[: i + 8])
                 break
             if marker_re.match(line) or not line:
                 # End of file, or start of next record
@@ -477,12 +506,16 @@ class UniprotRandomAccess(SequentialSeqFileRandomAccess):
         # TODO - Can we handle this directly in the parser?
         # This is a hack - use get_raw for <entry>...</entry> and wrap it with
         # the apparently required XML header and footer.
-        data = b"""<?xml version='1.0' encoding='UTF-8'?>
+        data = (
+            b"""<?xml version='1.0' encoding='UTF-8'?>
         <uniprot xmlns="http://uniprot.org/uniprot"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://uniprot.org/uniprot
         http://www.uniprot.org/support/docs/uniprot.xsd">
-        """ + self.get_raw(offset) + b"</uniprot>"
+        """
+            + self.get_raw(offset)
+            + b"</uniprot>"
+        )
         return next(SeqIO.UniprotIO.UniprotIterator(BytesIO(data)))
 
 
@@ -521,7 +554,7 @@ class IntelliGeneticsRandomAccess(SeqFileRandomAccess):
             while line and not line.startswith(b";"):
                 length += len(line)
                 line = handle.readline()
-            yield _bytes_to_string(key), offset, length
+            yield key.decode(), offset, length
             offset += length
             assert offset + len(line) == handle.tell()
 
@@ -563,7 +596,7 @@ class TabRandomAccess(SeqFileRandomAccess):
                 else:
                     raise err
             else:
-                yield _bytes_to_string(key), start_offset, len(line)
+                yield key.decode(), start_offset, len(line)
 
     def get_raw(self, offset):
         """Return the raw record from the file as a bytes string."""
@@ -576,6 +609,7 @@ class TabRandomAccess(SeqFileRandomAccess):
 # Now the FASTQ indexers #
 ##########################
 
+
 class FastqRandomAccess(SeqFileRandomAccess):
     """Random access to a FASTQ file (any supported variant).
 
@@ -619,7 +653,9 @@ class FastqRandomAccess(SeqFileRandomAccess):
                         # Special case, quality line should be just "\n"
                         line = handle.readline()
                         if line.strip():
-                            raise ValueError("Expected blank quality line, not %r" % line)
+                            raise ValueError(
+                                "Expected blank quality line, not %r" % line
+                            )
                         length += len(line)  # Need to include the blank ling
                     # Should be end of record...
                     end_offset = handle.tell()
@@ -633,7 +669,7 @@ class FastqRandomAccess(SeqFileRandomAccess):
                     length += len(line)
             if seq_len != qual_len:
                 raise ValueError("Problem with quality section")
-            yield _bytes_to_string(id), start_offset, length
+            yield id.decode(), start_offset, length
             start_offset = end_offset
         # print("EOF")
 
@@ -683,23 +719,24 @@ class FastqRandomAccess(SeqFileRandomAccess):
 
 ###############################################################################
 
-_FormatToRandomAccess = {"ace": SequentialSeqFileRandomAccess,
-                         "embl": EmblRandomAccess,
-                         "fasta": SequentialSeqFileRandomAccess,
-                         "fastq": FastqRandomAccess,  # Class handles all three variants
-                         "fastq-sanger": FastqRandomAccess,  # alias of the above
-                         "fastq-solexa": FastqRandomAccess,
-                         "fastq-illumina": FastqRandomAccess,
-                         "genbank": GenBankRandomAccess,
-                         "gb": GenBankRandomAccess,  # alias of the above
-                         "ig": IntelliGeneticsRandomAccess,
-                         "imgt": EmblRandomAccess,
-                         "phd": SequentialSeqFileRandomAccess,
-                         "pir": SequentialSeqFileRandomAccess,
-                         "sff": SffRandomAccess,
-                         "sff-trim": SffTrimedRandomAccess,
-                         "swiss": SwissRandomAccess,
-                         "tab": TabRandomAccess,
-                         "qual": SequentialSeqFileRandomAccess,
-                         "uniprot-xml": UniprotRandomAccess,
-                         }
+_FormatToRandomAccess = {
+    "ace": SequentialSeqFileRandomAccess,
+    "embl": EmblRandomAccess,
+    "fasta": SequentialSeqFileRandomAccess,
+    "fastq": FastqRandomAccess,  # Class handles all three variants
+    "fastq-sanger": FastqRandomAccess,  # alias of the above
+    "fastq-solexa": FastqRandomAccess,
+    "fastq-illumina": FastqRandomAccess,
+    "genbank": GenBankRandomAccess,
+    "gb": GenBankRandomAccess,  # alias of the above
+    "ig": IntelliGeneticsRandomAccess,
+    "imgt": EmblRandomAccess,
+    "phd": SequentialSeqFileRandomAccess,
+    "pir": SequentialSeqFileRandomAccess,
+    "sff": SffRandomAccess,
+    "sff-trim": SffTrimedRandomAccess,
+    "swiss": SwissRandomAccess,
+    "tab": TabRandomAccess,
+    "qual": SequentialSeqFileRandomAccess,
+    "uniprot-xml": UniprotRandomAccess,
+}
