@@ -1,6 +1,6 @@
 # Copyright 2000-2002 Andrew Dalke.  All rights reserved.
 # Copyright 2002-2004 Brad Chapman.  All rights reserved.
-# Copyright 2006-2017 by Peter Cock.  All rights reserved.
+# Copyright 2006-2020 by Peter Cock.  All rights reserved.
 #
 # This file is part of the Biopython distribution and governed by your
 # choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
@@ -9,13 +9,16 @@
 """Represent a Sequence Record, a sequence with annotation."""
 
 
-from Bio._py3k import basestring
-
 # NEEDS TO BE SYNCH WITH THE REST OF BIOPYTHON AND BIOPERL
 # In particular, the SeqRecord and BioSQL.BioSeq.DBSeqRecord classes
 # need to be in sync (this is the BioSQL "Database SeqRecord", see
 # also BioSQL.BioSeq.DBSeq which is the "Database Seq" class)
 
+
+from io import StringIO
+from Bio import StreamModeError
+
+
 _NO_SEQRECORD_COMPARISON = "SeqRecord comparison is deliberately not implemented. Explicitly compare the attributes of interest."
 
 
@@ -51,7 +54,7 @@ class _RestrictedDict(dict):
     pickled, for example for use in the multiprocessing library, we need to
     be able to pickle the restricted dictionary objects.
 
-    Using the default protocol, which is 0 on Python 2.x,
+    Using the default protocol, which is 3 on Python 3,
 
     >>> import pickle
     >>> y = pickle.loads(pickle.dumps(x))
@@ -60,7 +63,7 @@ class _RestrictedDict(dict):
     >>> y._length
     5
 
-    Using the highest protocol, which is 2 on Python 2.x,
+    Using the highest protocol, which is 4 on Python 3,
 
     >>> import pickle
     >>> z = pickle.loads(pickle.dumps(x, pickle.HIGHEST_PROTOCOL))
@@ -78,10 +81,15 @@ class _RestrictedDict(dict):
     def __setitem__(self, key, value):
         # The check hasattr(self, "_length") is to cope with pickle protocol 2
         # I couldn't seem to avoid this with __getstate__ and __setstate__
-        if not hasattr(value, "__len__") or not hasattr(value, "__getitem__") \
-                or (hasattr(self, "_length") and len(value) != self._length):
-            raise TypeError("We only allow python sequences (lists, tuples or "
-                            "strings) of length {0}.".format(self._length))
+        if (
+            not hasattr(value, "__len__")
+            or not hasattr(value, "__getitem__")
+            or (hasattr(self, "_length") and len(value) != self._length)
+        ):
+            raise TypeError(
+                "We only allow python sequences (lists, tuples or strings) "
+                f"of length {self._length}."
+            )
         dict.__setitem__(self, key, value)
 
     def update(self, new_dict):
@@ -90,7 +98,7 @@ class _RestrictedDict(dict):
             self[key] = value
 
 
-class SeqRecord(object):
+class SeqRecord:
     """A SeqRecord object holds a sequence and information about it.
 
     Main attributes:
@@ -151,10 +159,17 @@ class SeqRecord(object):
 
     """
 
-    def __init__(self, seq, id="<unknown id>", name="<unknown name>",
-                 description="<unknown description>", dbxrefs=None,
-                 features=None, annotations=None,
-                 letter_annotations=None):
+    def __init__(
+        self,
+        seq,
+        id="<unknown id>",
+        name="<unknown name>",
+        description="<unknown description>",
+        dbxrefs=None,
+        features=None,
+        annotations=None,
+        letter_annotations=None,
+    ):
         """Create a SeqRecord.
 
         Arguments:
@@ -183,12 +198,12 @@ class SeqRecord(object):
         You can create a 'blank' SeqRecord object, and then populate the
         attributes later.
         """
-        if id is not None and not isinstance(id, basestring):
+        if id is not None and not isinstance(id, str):
             # Lots of existing code uses id=None... this may be a bad idea.
             raise TypeError("id argument should be a string")
-        if not isinstance(name, basestring):
+        if not isinstance(name, str):
             raise TypeError("name argument should be a string")
-        if not isinstance(description, basestring):
+        if not isinstance(description, str):
             raise TypeError("description argument should be a string")
         self._seq = seq
         self.id = id
@@ -216,10 +231,11 @@ class SeqRecord(object):
                 self._per_letter_annotations = _RestrictedDict(length=0)
             else:
                 try:
-                    self._per_letter_annotations = \
-                        _RestrictedDict(length=len(seq))
+                    self._per_letter_annotations = _RestrictedDict(length=len(seq))
                 except TypeError:
-                    raise TypeError("seq argument should be a Seq object or similar")
+                    raise TypeError(
+                        "seq argument should be a Seq object or similar"
+                    ) from None
         else:
             # This will be handled via the property set function, which will
             # turn this into a _RestrictedDict and thus ensure all the values
@@ -230,14 +246,17 @@ class SeqRecord(object):
         if features is None:
             features = []
         elif not isinstance(features, list):
-            raise TypeError("features argument should be a list (of SeqFeature objects)")
+            raise TypeError(
+                "features argument should be a list (of SeqFeature objects)"
+            )
         self.features = features
 
     # TODO - Just make this a read only property?
     def _set_per_letter_annotations(self, value):
         if not isinstance(value, dict):
-            raise TypeError("The per-letter-annotations should be a "
-                            "(restricted) dictionary.")
+            raise TypeError(
+                "The per-letter-annotations should be a (restricted) dictionary."
+            )
         # Turn this into a restricted-dictionary (and check the entries)
         try:
             self._per_letter_annotations = _RestrictedDict(length=len(self.seq))
@@ -245,6 +264,7 @@ class SeqRecord(object):
             # e.g. seq is None
             self._per_letter_annotations = _RestrictedDict(length=0)
         self._per_letter_annotations.update(value)
+
     letter_annotations = property(
         fget=lambda self: self._per_letter_annotations,
         fset=_set_per_letter_annotations,
@@ -296,7 +316,8 @@ class SeqRecord(object):
 
         Note that if replacing the record's sequence with a sequence of a
         different length you must first clear the letter_annotations dict.
-        """)
+        """,
+    )
 
     def _set_seq(self, value):
         # TODO - Add a deprecation warning that the seq should be write only?
@@ -316,9 +337,11 @@ class SeqRecord(object):
                 # e.g. seq is None
                 self._per_letter_annotations = _RestrictedDict(length=0)
 
-    seq = property(fget=lambda self: self._seq,
-                   fset=_set_seq,
-                   doc="The sequence itself, as a Seq or MutableSeq object.")
+    seq = property(
+        fget=lambda self: self._seq,
+        fset=_set_seq,
+        doc="The sequence itself, as a Seq or MutableSeq object.",
+    )
 
     def __getitem__(self, index):
         """Return a sub-sequence or an individual letter.
@@ -441,20 +464,25 @@ class SeqRecord(object):
             parent_length = len(self)
             try:
                 from BioSQL.BioSeq import DBSeqRecord
+
                 biosql_available = True
             except ImportError:
                 biosql_available = False
 
             if biosql_available and isinstance(self, DBSeqRecord):
-                answer = SeqRecord(self.seq[index],
-                                   id=self.id,
-                                   name=self.name,
-                                   description=self.description)
+                answer = SeqRecord(
+                    self.seq[index],
+                    id=self.id,
+                    name=self.name,
+                    description=self.description,
+                )
             else:
-                answer = self.__class__(self.seq[index],
-                                        id=self.id,
-                                        name=self.name,
-                                        description=self.description)
+                answer = self.__class__(
+                    self.seq[index],
+                    id=self.id,
+                    name=self.name,
+                    description=self.description,
+                )
             # TODO - The description may no longer apply.
             # It would be safer to change it to something
             # generic like "edited" or the default value.
@@ -474,12 +502,17 @@ class SeqRecord(object):
                     if f.ref or f.ref_db:
                         # TODO - Implement this (with lots of tests)?
                         import warnings
-                        warnings.warn("When slicing SeqRecord objects, any "
-                                      "SeqFeature referencing other sequences (e.g. "
-                                      "from segmented GenBank records) are ignored.")
+
+                        warnings.warn(
+                            "When slicing SeqRecord objects, any "
+                            "SeqFeature referencing other sequences (e.g. "
+                            "from segmented GenBank records) are ignored."
+                        )
                         continue
-                    if start <= f.location.nofuzzy_start \
-                            and f.location.nofuzzy_end <= stop:
+                    if (
+                        start <= f.location.nofuzzy_start
+                        and f.location.nofuzzy_end <= stop
+                    ):
                         answer.features.append(f._shift(-start))
 
             # Slice all the values to match the sliced sequence
@@ -610,18 +643,20 @@ class SeqRecord(object):
         """
         lines = []
         if self.id:
-            lines.append("ID: {0}".format(self.id))
+            lines.append(f"ID: {self.id}")
         if self.name:
-            lines.append("Name: {0}".format(self.name))
+            lines.append(f"Name: {self.name}")
         if self.description:
-            lines.append("Description: {0}".format(self.description))
+            lines.append(f"Description: {self.description}")
         if self.dbxrefs:
             lines.append("Database cross-references: " + ", ".join(self.dbxrefs))
-        lines.append("Number of features: {0}".format(len(self.features)))
+        lines.append(f"Number of features: {len(self.features)}")
         for a in self.annotations:
-            lines.append("/{0}={1}".format(a, str(self.annotations[a])))
+            lines.append(f"/{a}={str(self.annotations[a])}")
         if self.letter_annotations:
-            lines.append("Per letter annotation for: " + ", ".join(self.letter_annotations))
+            lines.append(
+                "Per letter annotation for: " + ", ".join(self.letter_annotations)
+            )
         # Don't want to include the entire sequence,
         # and showing the alphabet is useful:
         lines.append(repr(self.seq))
@@ -656,10 +691,11 @@ class SeqRecord(object):
         annotations, letter_annotations and features are not shown (as they
         would lead to a very long string).
         """
-        return "{0}(seq={1!r}, id={2!r}, name={3!r}, description={4!r}, dbxrefs={5!r})".format(
-               self.__class__.__name__,
-               self.seq, self.id, self.name,
-               self.description, self.dbxrefs)
+        return (
+            f"{self.__class__.__name__}(seq={self.seq!r}, id={self.id!r},"
+            f" name={self.name!r}, description={self.description!r},"
+            f" dbxrefs={self.dbxrefs!r})"
+        )
 
     def format(self, format):
         r"""Return the record as a string in the specified file format.
@@ -682,29 +718,43 @@ class SeqRecord(object):
         MKQHKAMIVALIVICITAVVAALVTRKDLCEVHIRTGQTEVAVF
         <BLANKLINE>
 
-        The python print command automatically appends a new line, meaning
+        The Python print function automatically appends a new line, meaning
         in this example a blank line is shown.  If you look at the string
         representation you can see there is a trailing new line (shown as
         slash n) which is important when writing to a file or if
         concatenating multiple sequence strings together.
 
         Note that this method will NOT work on every possible file format
-        supported by Bio.SeqIO (e.g. some are for multiple sequences only).
+        supported by Bio.SeqIO (e.g. some are for multiple sequences only,
+        and binary formats are not supported).
         """
-        # See also the __format__ added for Python 2.6 / 3.0, PEP 3101
+        # See also the __format__ method
         # See also the Bio.Align.Generic.Alignment class and its format()
         return self.__format__(format)
 
     def __format__(self, format_spec):
-        """Return the record as a string in the specified file format.
+        r"""Return the record as a string in the specified file format.
 
-        This method supports the python format() function added in
-        Python 2.6/3.0.  The format_spec should be a lower case string
-        supported by Bio.SeqIO as an output file format. See also the
-        SeqRecord's format() method.
+        This method supports the Python format() function and f-strings.
+        The format_spec should be a lower case string supported by
+        Bio.SeqIO as a text output file format. Requesting a binary file
+        format raises a ValueError. e.g.
 
-        Under Python 3 please note that for binary formats a bytes
-        string is returned, otherwise a (unicode) string is returned.
+        >>> from Bio.Seq import Seq
+        >>> from Bio.SeqRecord import SeqRecord
+        >>> record = SeqRecord(Seq("MKQHKAMIVALIVICITAVVAALVTRKDLCEVHIRTGQTEVAVF"),
+        ...                    id="YP_025292.1", name="HokC",
+        ...                    description="toxic membrane protein")
+        ...
+        >>> format(record, "fasta")
+        '>YP_025292.1 toxic membrane protein\nMKQHKAMIVALIVICITAVVAALVTRKDLCEVHIRTGQTEVAVF\n'
+        >>> print(f"Here is {record.id} in FASTA format:\n{record:fasta}")
+        Here is YP_025292.1 in FASTA format:
+        >YP_025292.1 toxic membrane protein
+        MKQHKAMIVALIVICITAVVAALVTRKDLCEVHIRTGQTEVAVF
+        <BLANKLINE>
+
+        See also the SeqRecord's format() method.
         """
         if not format_spec:
             # Follow python convention and default to using __str__
@@ -716,14 +766,14 @@ class SeqRecord(object):
             return SeqIO._FormatToString[format_spec](self)
 
         # Harder case, make a temp handle instead
-        if format_spec in SeqIO._BinaryFormats:
-            # Return bytes on Python 3
-            from io import BytesIO
-            handle = BytesIO()
-        else:
-            from Bio._py3k import StringIO
-            handle = StringIO()
-        SeqIO.write(self, handle, format_spec)
+        handle = StringIO()
+        try:
+            SeqIO.write(self, handle, format_spec)
+        except StreamModeError:
+            raise ValueError(
+                "Binary format %s cannot be used with SeqRecord format method"
+                % format_spec
+            ) from None
         return handle.getvalue()
 
     def __len__(self):
@@ -764,14 +814,6 @@ class SeqRecord(object):
         """Define the greater-than-or-equal-to operand (not implemented)."""
         raise NotImplementedError(_NO_SEQRECORD_COMPARISON)
 
-    # Make SeqRecord unhashable explicit, required for Python 2.
-    # See github issue 929 for related discussion.
-    __hash__ = None
-
-    # Note Python 3 does not use __cmp__ and there is no need to
-    # define __cmp__ on Python 2 as have all of  _lt__ etc defined.
-
-    # Python 3:
     def __bool__(self):
         """Boolean value of an instance of this class (True).
 
@@ -787,9 +829,6 @@ class SeqRecord(object):
         """
         return True
 
-    # Python 2:
-    __nonzero__ = __bool__
-
     def __add__(self, other):
         """Add another sequence or string to this sequence.
 
@@ -863,16 +902,19 @@ class SeqRecord(object):
         if not isinstance(other, SeqRecord):
             # Assume it is a string or a Seq.
             # Note can't transfer any per-letter-annotations
-            return SeqRecord(self.seq + other,
-                             id=self.id, name=self.name,
-                             description=self.description,
-                             features=self.features[:],
-                             annotations=self.annotations.copy(),
-                             dbxrefs=self.dbxrefs[:])
+            return SeqRecord(
+                self.seq + other,
+                id=self.id,
+                name=self.name,
+                description=self.description,
+                features=self.features[:],
+                annotations=self.annotations.copy(),
+                dbxrefs=self.dbxrefs[:],
+            )
         # Adding two SeqRecord objects... must merge annotation.
-        answer = SeqRecord(self.seq + other.seq,
-                           features=self.features[:],
-                           dbxrefs=self.dbxrefs[:])
+        answer = SeqRecord(
+            self.seq + other.seq, features=self.features[:], dbxrefs=self.dbxrefs[:]
+        )
         # Will take all the features and all the db cross refs,
         length = len(self)
         for f in other.features:
@@ -918,17 +960,22 @@ class SeqRecord(object):
         []
         """
         if isinstance(other, SeqRecord):
-            raise RuntimeError("This should have happened via the __add__ of "
-                               "the other SeqRecord being added!")
+            raise RuntimeError(
+                "This should have happened via the __add__ of "
+                "the other SeqRecord being added!"
+            )
         # Assume it is a string or a Seq.
         # Note can't transfer any per-letter-annotations
         offset = len(other)
-        return SeqRecord(other + self.seq,
-                         id=self.id, name=self.name,
-                         description=self.description,
-                         features=[f._shift(offset) for f in self.features],
-                         annotations=self.annotations.copy(),
-                         dbxrefs=self.dbxrefs[:])
+        return SeqRecord(
+            other + self.seq,
+            id=self.id,
+            name=self.name,
+            description=self.description,
+            features=[f._shift(offset) for f in self.features],
+            annotations=self.annotations.copy(),
+            dbxrefs=self.dbxrefs[:],
+        )
 
     def upper(self):
         """Return a copy of the record with an upper case sequence.
@@ -957,13 +1004,16 @@ class SeqRecord(object):
         "#$%&'()
         <BLANKLINE>
         """
-        return SeqRecord(self.seq.upper(),
-                         id=self.id, name=self.name,
-                         description=self.description,
-                         dbxrefs=self.dbxrefs[:],
-                         features=self.features[:],
-                         annotations=self.annotations.copy(),
-                         letter_annotations=self.letter_annotations.copy())
+        return SeqRecord(
+            self.seq.upper(),
+            id=self.id,
+            name=self.name,
+            description=self.description,
+            dbxrefs=self.dbxrefs[:],
+            features=self.features[:],
+            annotations=self.annotations.copy(),
+            letter_annotations=self.letter_annotations.copy(),
+        )
 
     def lower(self):
         """Return a copy of the record with a lower case sequence.
@@ -997,17 +1047,27 @@ class SeqRecord(object):
         >>> old.dbxrefs == new.dbxrefs
         True
         """
-        return SeqRecord(self.seq.lower(),
-                         id=self.id, name=self.name,
-                         description=self.description,
-                         dbxrefs=self.dbxrefs[:],
-                         features=self.features[:],
-                         annotations=self.annotations.copy(),
-                         letter_annotations=self.letter_annotations.copy())
+        return SeqRecord(
+            self.seq.lower(),
+            id=self.id,
+            name=self.name,
+            description=self.description,
+            dbxrefs=self.dbxrefs[:],
+            features=self.features[:],
+            annotations=self.annotations.copy(),
+            letter_annotations=self.letter_annotations.copy(),
+        )
 
-    def reverse_complement(self, id=False, name=False, description=False,
-                           features=True, annotations=False,
-                           letter_annotations=True, dbxrefs=False):
+    def reverse_complement(
+        self,
+        id=False,
+        name=False,
+        description=False,
+        features=True,
+        annotations=False,
+        letter_annotations=True,
+        dbxrefs=False,
+    ):
         """Return new SeqRecord with reverse complement sequence.
 
         By default the new record does NOT preserve the sequence identifier,
@@ -1135,20 +1195,21 @@ class SeqRecord(object):
         Test ACGA
         """
         from Bio.Seq import MutableSeq  # Lazy to avoid circular imports
+
         if isinstance(self.seq, MutableSeq):
             # Currently the MutableSeq reverse complement is in situ
             answer = SeqRecord(self.seq.toseq().reverse_complement())
         else:
             answer = SeqRecord(self.seq.reverse_complement())
-        if isinstance(id, basestring):
+        if isinstance(id, str):
             answer.id = id
         elif id:
             answer.id = self.id
-        if isinstance(name, basestring):
+        if isinstance(name, str):
             answer.name = name
         elif name:
             answer.name = self.name
-        if isinstance(description, basestring):
+        if isinstance(description, str):
             answer.description = description
         elif description:
             answer.description = self.description
@@ -1182,14 +1243,23 @@ class SeqRecord(object):
                 answer._per_letter_annotations[key] = value[::-1]
         return answer
 
-    def translate(self,
-                  # Seq translation arguments:
-                  table="Standard", stop_symbol="*", to_stop=False,
-                  cds=False, gap=None,
-                  # SeqRecord annotation arguments:
-                  id=False, name=False, description=False,
-                  features=False, annotations=False,
-                  letter_annotations=False, dbxrefs=False):
+    def translate(
+        self,
+        # Seq translation arguments:
+        table="Standard",
+        stop_symbol="*",
+        to_stop=False,
+        cds=False,
+        gap=None,
+        # SeqRecord annotation arguments:
+        id=False,
+        name=False,
+        description=False,
+        features=False,
+        annotations=False,
+        letter_annotations=False,
+        dbxrefs=False,
+    ):
         """Return new SeqRecord with translated sequence.
 
         This calls the record's .seq.translate() method (which describes
@@ -1235,20 +1305,20 @@ class SeqRecord(object):
         <BLANKLINE>
 
         """
-        answer = SeqRecord(self.seq.translate(table=table,
-                                              stop_symbol=stop_symbol,
-                                              to_stop=to_stop,
-                                              cds=cds,
-                                              gap=gap))
-        if isinstance(id, basestring):
+        answer = SeqRecord(
+            self.seq.translate(
+                table=table, stop_symbol=stop_symbol, to_stop=to_stop, cds=cds, gap=gap
+            )
+        )
+        if isinstance(id, str):
             answer.id = id
         elif id:
             answer.id = self.id
-        if isinstance(name, basestring):
+        if isinstance(name, str):
             answer.name = name
         elif name:
             answer.name = self.name
-        if isinstance(description, basestring):
+        if isinstance(description, str):
             answer.description = description
         elif description:
             answer.description = self.description
@@ -1271,10 +1341,13 @@ class SeqRecord(object):
             answer.letter_annotations = letter_annotations
         elif letter_annotations:
             # Does not make sense to copy these as length now wrong
-            raise TypeError("Unexpected letter_annotations argument %r" % letter_annotations)
+            raise TypeError(
+                "Unexpected letter_annotations argument %r" % letter_annotations
+            )
         return answer
 
 
 if __name__ == "__main__":
     from Bio._utils import run_doctest
+
     run_doctest()
