@@ -5,13 +5,12 @@
 # as part of this package.
 
 """Code for dealing with Codon Alignments."""
-from __future__ import print_function
+
+from collections.abc import Mapping, Iterable
 
 from Bio import BiopythonWarning
 from Bio import BiopythonExperimentalWarning
 
-from Bio._py3k import zip
-
 from Bio.Alphabet import _get_base_alphabet
 from Bio.SeqRecord import SeqRecord
 
@@ -22,14 +21,26 @@ from Bio.codonalign.codonalphabet import default_codon_table, default_codon_alph
 from Bio.codonalign.codonalphabet import get_codon_alphabet as _get_codon_alphabet
 
 import warnings
-warnings.warn('Bio.codonalign is an experimental module which may undergo '
-              'significant changes prior to its future official release.',
-              BiopythonExperimentalWarning)
+
+warnings.warn(
+    "Bio.codonalign is an experimental module which may undergo "
+    "significant changes prior to its future official release.",
+    BiopythonExperimentalWarning,
+)
 
 
-def build(pro_align, nucl_seqs, corr_dict=None, gap_char='-', unknown='X',
-          codon_table=default_codon_table, alphabet=None,
-          complete_protein=False, anchor_len=10, max_score=10):
+def build(
+    pro_align,
+    nucl_seqs,
+    corr_dict=None,
+    gap_char="-",
+    unknown="X",
+    codon_table=default_codon_table,
+    alphabet=None,
+    complete_protein=False,
+    anchor_len=10,
+    max_score=10,
+):
     """Build a codon alignment from protein alignment and corresponding nucleotides.
 
     Arguments:
@@ -70,27 +81,31 @@ def build(pro_align, nucl_seqs, corr_dict=None, gap_char='-', unknown='X',
 
     # check the type of object of pro_align
     if not isinstance(pro_align, MultipleSeqAlignment):
-        raise TypeError("the first argument should be a MultipleSeqAlignment "
-                        "object")
+        raise TypeError("the first argument should be a MultipleSeqAlignment object")
     # check the alphabet of pro_align
     for pro in pro_align:
         if not isinstance(_get_base_alphabet(pro.seq.alphabet), ProteinAlphabet):
-            raise TypeError("Alphabet Error!\nThe input alignment should be "
-                            "a *PROTEIN* alignemnt, found %r" % pro.seq.alphabet)
+            raise TypeError(
+                "Alphabet Error!\nThe input alignment should be "
+                "a *PROTEIN* alignment, found %r" % pro.seq.alphabet
+            )
     if alphabet is None:
         alphabet = _get_codon_alphabet(codon_table, gap_char=gap_char)
     # check whether the number of seqs in pro_align and nucl_seqs is
     # the same
     pro_num = len(pro_align)
     if corr_dict is None:
-        if nucl_seqs.__class__.__name__ == "generator":
-            # nucl_seqs will be a tuple if read by SeqIO.parse()
+        try:
+            nucl_num = len(nucl_seqs)
+        except TypeError:
+            # nucl_seqs will be an iterator if returned by SeqIO.parse()
             nucl_seqs = tuple(nucl_seqs)
-        nucl_num = len(nucl_seqs)
+            nucl_num = len(nucl_seqs)
         if pro_num > nucl_num:
-            raise ValueError("Higher Number of SeqRecords in Protein Alignment "
-                             "({0}) than the Number of Nucleotide SeqRecords "
-                             "({1}) are found!".format(pro_num, nucl_num))
+            raise ValueError(
+                f"Higher Number of SeqRecords in Protein Alignment ({pro_num}) "
+                f"than the Number of Nucleotide SeqRecords ({nucl_num}) are found!"
+            )
 
         # Determine the protein sequences and nucl sequences
         # correspondence. If nucl_seqs is a list, tuple or read by
@@ -98,35 +113,37 @@ def build(pro_align, nucl_seqs, corr_dict=None, gap_char='-', unknown='X',
         # and nucl_seqs are the same. If nucl_seqs is a dict or read by
         # SeqIO.index(), we match seqs in pro_align and those in
         # nucl_seq by their id.
-        if nucl_seqs.__class__.__name__ in ("_IndexedSeqFileDict", "dict"):
+        if isinstance(nucl_seqs, Mapping):
             corr_method = 1
-        elif nucl_seqs.__class__.__name__ in ("list", "tuple"):
+        elif isinstance(nucl_seqs, Iterable):
             corr_method = 0
         else:
-            raise TypeError("Nucl Sequences Error, Unknown type to assign "
-                            "correspondence method")
+            raise TypeError(
+                "Nucl Sequences Error, Unknown type to assign correspondence method"
+            )
     else:
         if not isinstance(corr_dict, dict):
-            raise TypeError("corr_dict should be a dict that corresponds "
-                            "protein id to nucleotide id!")
+            raise TypeError(
+                "corr_dict should be a dict that corresponds "
+                "protein id to nucleotide id!"
+            )
         if len(corr_dict) >= pro_num:
-            # read by SeqIO.parse()
-            if nucl_seqs.__class__.__name__ == "generator":
-                from Bio import SeqIO
-                nucl_seqs = SeqIO.to_dict(nucl_seqs)
-            elif nucl_seqs.__class__.__name__ in ("list", "tuple"):
-                nucl_seqs = {i.id: i for i in nucl_seqs}
-            elif nucl_seqs.__class__.__name__ in \
-                    ("_IndexedSeqFileDict", "dict"):
+            if isinstance(nucl_seqs, Mapping):
                 pass
             else:
-                raise TypeError("Nucl Sequences Error, Unknown type of "
-                                "Nucleotide Records!")
+                d = {}
+                for record in nucl_seqs:
+                    key = record.id
+                    if key in d:
+                        raise ValueError("Duplicate key '%s'" % key)
+                    d[key] = record
+                nucl_seqs = d
             corr_method = 2
         else:
-            raise RuntimeError("Number of items in corr_dict ({0}) is less "
-                               "than number of protein records "
-                               "({1})".format(len(corr_dict), pro_num))
+            raise RuntimeError(
+                f"Number of items in corr_dict ({len(corr_dict)}) "
+                f"is less than number of protein records ({pro_num})"
+            )
 
     # set up pro-nucl correspondence based on corr_method
     # corr_method = 0, consecutive pairing
@@ -139,9 +156,10 @@ def build(pro_align, nucl_seqs, corr_dict=None, gap_char='-', unknown='X',
         # check if there is pro_id that does not have a nucleotide match
         if pro_id - nucl_id:
             diff = pro_id - nucl_id
-            raise ValueError("Protein Record {0} cannot find a nucleotide "
-                             "sequence match, please check the "
-                             "id".format(', '.join(diff)))
+            raise ValueError(
+                f"Protein Record {', '.join(diff)} cannot find a "
+                "nucleotide sequence match, please check the id"
+            )
         else:
             pro_nucl_pair = []
             for pro_rec in pro_align:
@@ -162,19 +180,29 @@ def build(pro_align, nucl_seqs, corr_dict=None, gap_char='-', unknown='X',
     for pair in pro_nucl_pair:
         # Beware that the following span corresponds to an ungapped
         # nucleotide sequence.
-        corr_span = _check_corr(pair[0], pair[1], gap_char=gap_char,
-                                codon_table=codon_table,
-                                complete_protein=complete_protein,
-                                anchor_len=anchor_len)
+        corr_span = _check_corr(
+            pair[0],
+            pair[1],
+            gap_char=gap_char,
+            codon_table=codon_table,
+            complete_protein=complete_protein,
+            anchor_len=anchor_len,
+        )
         if not corr_span:
-            raise ValueError("Protein Record {0} and Nucleotide Record {1} do"
-                             " not match!".format(pair[0].id, pair[1].id))
+            raise ValueError(
+                f"Protein Record {pair[0].id} and "
+                f"Nucleotide Record {pair[1].id} do not match!"
+            )
         else:
-            codon_rec = _get_codon_rec(pair[0], pair[1], corr_span,
-                                       alphabet=alphabet,
-                                       complete_protein=False,
-                                       codon_table=codon_table,
-                                       max_score=max_score)
+            codon_rec = _get_codon_rec(
+                pair[0],
+                pair[1],
+                corr_span,
+                alphabet=alphabet,
+                complete_protein=False,
+                codon_table=codon_table,
+                max_score=max_score,
+            )
             codon_aln.append(codon_rec)
             if corr_span[1] == 2:
                 shift = True
@@ -186,16 +214,16 @@ def build(pro_align, nucl_seqs, corr_dict=None, gap_char='-', unknown='X',
 
 def _codons2re(codons):
     """Generate regular expression based on a given list of codons (PRIVATE)."""
-    reg = ''
+    reg = ""
     for i in zip(*codons):
         if len(set(i)) == 1:
-            reg += ''.join(set(i))
+            reg += "".join(set(i))
         else:
-            reg += '[' + ''.join(set(i)) + ']'
+            reg += "[" + "".join(set(i)) + "]"
     return reg
 
 
-def _get_aa_regex(codon_table, stop='*', unknown='X'):
+def _get_aa_regex(codon_table, stop="*", unknown="X"):
     """Set up the regular expression of a given CodonTable (PRIVATE).
 
     >>> from Bio.Data.CodonTable import generic_by_id
@@ -216,21 +244,27 @@ def _get_aa_regex(codon_table, stop='*', unknown='X'):
 
     """
     from Bio.Data.CodonTable import CodonTable
+
     if not isinstance(codon_table, CodonTable):
-        raise TypeError("Input table is not a instance of "
-                        "Bio.Data.CodonTable object")
+        raise TypeError("Input table is not a instance of Bio.Data.CodonTable object")
     aa2codon = {}
     for codon, aa in codon_table.forward_table.items():
         aa2codon.setdefault(aa, []).append(codon)
     for aa, codons in aa2codon.items():
         aa2codon[aa] = _codons2re(codons)
     aa2codon[stop] = _codons2re(codon_table.stop_codons)
-    aa2codon[unknown] = '...'
+    aa2codon[unknown] = "..."
     return aa2codon
 
 
-def _check_corr(pro, nucl, gap_char='-', codon_table=default_codon_table,
-                complete_protein=False, anchor_len=10):
+def _check_corr(
+    pro,
+    nucl,
+    gap_char="-",
+    codon_table=default_codon_table,
+    complete_protein=False,
+    anchor_len=10,
+):
     """Check if the nucleotide can be translated into the protein (PRIVATE).
 
     Expects two SeqRecord objects.
@@ -239,20 +273,23 @@ def _check_corr(pro, nucl, gap_char='-', codon_table=default_codon_table,
     from Bio.Alphabet import NucleotideAlphabet
 
     if not isinstance(pro, SeqRecord) or not isinstance(nucl, SeqRecord):
-        raise TypeError("_check_corr accepts two SeqRecord object. Please "
-                        "check your input.")
+        raise TypeError(
+            "_check_corr accepts two SeqRecord object. Please check your input."
+        )
 
     def get_alpha(alpha):
-        if hasattr(alpha, 'alphabet'):
+        if hasattr(alpha, "alphabet"):
             return get_alpha(alpha.alphabet)
         else:
             return alpha
 
-    if not isinstance(_get_base_alphabet(get_alpha(nucl.seq.alphabet)),
-                      NucleotideAlphabet):
-        raise TypeError("Alphabet for nucl should be an instance of "
-                        "NucleotideAlphabet, {0} "
-                        "detected".format(str(nucl.seq.alphabet)))
+    if not isinstance(
+        _get_base_alphabet(get_alpha(nucl.seq.alphabet)), NucleotideAlphabet
+    ):
+        raise TypeError(
+            "Alphabet for nucl should be an instance of NucleotideAlphabet, "
+            f"{str(nucl.seq.alphabet)} detected"
+        )
 
     aa2re = _get_aa_regex(codon_table)
     pro_re = ""
@@ -270,8 +307,9 @@ def _check_corr(pro, nucl, gap_char='-', codon_table=default_codon_table,
         # have a try
         # anchor_len = 10 # adjust this value to test performance
         pro_seq = str(pro.seq).replace(gap_char, "")
-        anchors = [pro_seq[i:(i + anchor_len)] for i in
-                   range(0, len(pro_seq), anchor_len)]
+        anchors = [
+            pro_seq[i : (i + anchor_len)] for i in range(0, len(pro_seq), anchor_len)
+        ]
         # if the last anchor is less than the specified anchor
         # size, we combine the penultimate and the last anchor
         # together as the last one.
@@ -295,17 +333,17 @@ def _check_corr(pro, nucl, gap_char='-', codon_table=default_codon_table,
                 for aa in anchor:
                     if complete_protein and i == 0:
                         qcodon += _codons2re(codon_table.start_codons)
-                        fncodon += aa2re['X']
+                        fncodon += aa2re["X"]
                         continue
                     qcodon += aa2re[aa]
-                    fncodon += aa2re['X']
+                    fncodon += aa2re["X"]
                 match = re.search(qcodon, nucl_seq)
             elif this_anchor_len > anchor_len:
                 last_qcodon = ""
                 last_fcodon = ""
                 for j in range(anchor_len, len(anchor)):
                     last_qcodon += aa2re[anchor[j]]
-                    last_fcodon += aa2re['X']
+                    last_fcodon += aa2re["X"]
                 match = re.search(last_qcodon, nucl_seq)
             # build full_pro_re from anchors
             if match:
@@ -342,86 +380,99 @@ def _check_corr(pro, nucl, gap_char='-', codon_table=default_codon_table,
                     elif shift_val in (3 * anchor_len - 2, 3 * anchor_len - 1):
                         sh_nuc_len = anchor_len * 3 - (3 * anchor_len - shift_val)
                     if anchor_pos[0][0] >= sh_nuc_len:
-                        sh_nuc = nucl_seq[anchor_pos[0][0] - sh_nuc_len:anchor_pos[0][0]]
+                        sh_nuc = nucl_seq[
+                            anchor_pos[0][0] - sh_nuc_len : anchor_pos[0][0]
+                        ]
                     else:
                         # this is unlikely to produce the correct output
-                        sh_nuc = nucl_seq[:anchor_pos[0][0]]
-                    qcodon, shift_id_pos = _get_shift_anchor_re(sh_anc, sh_nuc,
-                                                                shift_val,
-                                                                aa2re,
-                                                                anchor_len,
-                                                                shift_id_pos)
+                        sh_nuc = nucl_seq[: anchor_pos[0][0]]
+                    qcodon, shift_id_pos = _get_shift_anchor_re(
+                        sh_anc, sh_nuc, shift_val, aa2re, anchor_len, shift_id_pos
+                    )
                     if qcodon is not None and qcodon != -1:
                         # pro_re[0] should be '.'*anchor_len, therefore I
                         # replace it.
                         pro_re[0] = qcodon
                         break
                 if qcodon == -1:
-                    warnings.warn("first frameshift detection failed for "
-                                  "{0}".format(nucl.id), BiopythonWarning)
+                    warnings.warn(
+                        f"first frameshift detection failed for {nucl.id}",
+                        BiopythonWarning,
+                    )
             # check anchors in the middle
             for i in range(len(anchor_pos) - 1):
-                shift_val = (anchor_pos[i + 1][0] - anchor_pos[i][0]) % \
-                            (3 * anchor_len)
-                sh_anc = "".join(anchors[anchor_pos[i][2]:anchor_pos[i + 1][2]])
-                sh_nuc = nucl_seq[anchor_pos[i][0]:anchor_pos[i + 1][0]]
+                shift_val = (anchor_pos[i + 1][0] - anchor_pos[i][0]) % (3 * anchor_len)
+                sh_anc = "".join(anchors[anchor_pos[i][2] : anchor_pos[i + 1][2]])
+                sh_nuc = nucl_seq[anchor_pos[i][0] : anchor_pos[i + 1][0]]
                 qcodon = None
                 if shift_val != 0:
-                    qcodon, shift_id_pos = _get_shift_anchor_re(sh_anc, sh_nuc,
-                                                                shift_val,
-                                                                aa2re,
-                                                                anchor_len,
-                                                                shift_id_pos)
+                    qcodon, shift_id_pos = _get_shift_anchor_re(
+                        sh_anc, sh_nuc, shift_val, aa2re, anchor_len, shift_id_pos
+                    )
                 if qcodon is not None and qcodon != -1:
-                    pro_re[anchor_pos[i][2]:anchor_pos[i + 1][2]] = [qcodon]
+                    pro_re[anchor_pos[i][2] : anchor_pos[i + 1][2]] = [qcodon]
                     qcodon = None
                 elif qcodon == -1:
-                    warnings.warn("middle frameshift detection failed for "
-                                  "{0}".format(nucl.id), BiopythonWarning)
+                    warnings.warn(
+                        f"middle frameshift detection failed for {nucl.id}",
+                        BiopythonWarning,
+                    )
             # check the last anchor
             if anchor_pos[-1][2] + 1 == len(anchors) - 1:
                 sh_anc = anchors[-1]
                 this_anchor_len = len(sh_anc)
-                shift_val_lst = [1, 2, 3 * this_anchor_len - 2, 3 * this_anchor_len - 1, 0]
+                shift_val_lst = [
+                    1,
+                    2,
+                    3 * this_anchor_len - 2,
+                    3 * this_anchor_len - 1,
+                    0,
+                ]
                 for shift_val in shift_val_lst:
                     if shift_val == 0:
                         qcodon = None
                         break
                     if shift_val in (1, 2):
                         sh_nuc_len = this_anchor_len * 3 + shift_val
-                    elif shift_val in \
-                            (3 * this_anchor_len - 2, 3 * this_anchor_len - 1):
-                        sh_nuc_len = this_anchor_len * 3 - (3 * this_anchor_len - shift_val)
+                    elif shift_val in (
+                        3 * this_anchor_len - 2,
+                        3 * this_anchor_len - 1,
+                    ):
+                        sh_nuc_len = this_anchor_len * 3 - (
+                            3 * this_anchor_len - shift_val
+                        )
                     if len(nucl_seq) - anchor_pos[-1][0] >= sh_nuc_len:
-                        sh_nuc = nucl_seq[anchor_pos[-1][0]:anchor_pos[-1][0] + sh_nuc_len]
+                        sh_nuc = nucl_seq[
+                            anchor_pos[-1][0] : anchor_pos[-1][0] + sh_nuc_len
+                        ]
                     else:
                         # this is unlikely to produce the correct output
-                        sh_nuc = nucl_seq[anchor_pos[-1][0]:]
-                    qcodon, shift_id_pos = _get_shift_anchor_re(sh_anc, sh_nuc,
-                                                                shift_val,
-                                                                aa2re,
-                                                                this_anchor_len,
-                                                                shift_id_pos)
+                        sh_nuc = nucl_seq[anchor_pos[-1][0] :]
+                    qcodon, shift_id_pos = _get_shift_anchor_re(
+                        sh_anc, sh_nuc, shift_val, aa2re, this_anchor_len, shift_id_pos
+                    )
                     if qcodon is not None and qcodon != -1:
                         pro_re.pop()
                         pro_re[-1] = qcodon
                         break
                 if qcodon == -1:
-                    warnings.warn("last frameshift detection failed for "
-                                  "{0}".format(nucl.id), BiopythonWarning)
+                    warnings.warn(
+                        f"last frameshift detection failed for {nucl.id}",
+                        BiopythonWarning,
+                    )
             # try global match
             full_pro_re = "".join(pro_re)
             match = re.search(full_pro_re, nucl_seq)
             if match:
                 return (match.span(), 2, match)
             else:
-                raise RuntimeError("Protein SeqRecord ({0}) and Nucleotide "
-                                   "SeqRecord ({1}) do not "
-                                   "match!".format(pro.id, nucl.id))
+                raise RuntimeError(
+                    f"Protein SeqRecord ({pro.id}) and "
+                    f"Nucleotide SeqRecord ({nucl.id}) do not match!"
+                )
 
 
-def _get_shift_anchor_re(sh_anc, sh_nuc, shift_val, aa2re, anchor_len,
-                         shift_id_pos):
+def _get_shift_anchor_re(sh_anc, sh_nuc, shift_val, aa2re, anchor_len, shift_id_pos):
     """Find a regular expression matching a potentially shifted anchor (PRIVATE).
 
     Arguments:
@@ -437,6 +488,7 @@ def _get_shift_anchor_re(sh_anc, sh_nuc, shift_val, aa2re, anchor_len,
 
     """
     import re
+
     shift_id = [chr(i) for i in range(97, 107)]
     if 0 < shift_val < 3 * anchor_len - 2:
         # if shift_val in (1, 2):
@@ -450,7 +502,7 @@ def _get_shift_anchor_re(sh_anc, sh_nuc, shift_val, aa2re, anchor_len,
             qcodon += "$"
             match = re.search(qcodon, sh_nuc)
             if match:
-                qcodon = qcodon.replace('^', '').replace('$', '')
+                qcodon = qcodon.replace("^", "").replace("$", "")
                 shift_id_pos += 1
                 return qcodon, shift_id_pos
         if not match:
@@ -469,14 +521,18 @@ def _get_shift_anchor_re(sh_anc, sh_nuc, shift_val, aa2re, anchor_len,
                     pass
                 elif k == j:
                     qcodon += _merge_aa2re(
-                            sh_anc[j - 1], sh_anc[j], shift_val, aa2re,
-                            shift_id[shift_id_pos].upper())
+                        sh_anc[j - 1],
+                        sh_anc[j],
+                        shift_val,
+                        aa2re,
+                        shift_id[shift_id_pos].upper(),
+                    )
                 else:
                     qcodon += aa2re[aa]
-            qcodon += '$'
+            qcodon += "$"
             match = re.search(qcodon, sh_nuc)
             if match:
-                qcodon = qcodon.replace('^', '').replace('$', '')
+                qcodon = qcodon.replace("^", "").replace("$", "")
                 shift_id_pos += 1
                 return qcodon, shift_id_pos
         if not match:
@@ -486,14 +542,15 @@ def _get_shift_anchor_re(sh_anc, sh_nuc, shift_val, aa2re, anchor_len,
 
 def _merge_aa2re(aa1, aa2, shift_val, aa2re, reid):
     """Merge two amino acids based on detected frame shift value (PRIVATE)."""
+
     def get_aa_from_codonre(re_aa):
         aas = []
         m = 0
         for i in re_aa:
-            if i == '[':
+            if i == "[":
                 m = -1
-                aas.append('')
-            elif i == ']':
+                aas.append("")
+            elif i == "]":
                 m = 0
                 continue
             elif m == -1:
@@ -501,30 +558,60 @@ def _merge_aa2re(aa1, aa2, shift_val, aa2re, reid):
             elif m == 0:
                 aas.append(i)
         return aas
+
     scodon = list(map(get_aa_from_codonre, (aa2re[aa1], aa2re[aa2])))
     if shift_val == 1:
-        intersect = ''.join(set(scodon[0][2]) & set(scodon[1][0]))
-        scodonre = '(?P<' + reid + '>'
-        scodonre += '[' + scodon[0][0] + ']' + \
-                    '[' + scodon[0][1] + ']' + \
-                    '[' + intersect + ']' + \
-                    '[' + scodon[1][1] + ']' + \
-                    '[' + scodon[1][2] + ']'
+        intersect = "".join(set(scodon[0][2]) & set(scodon[1][0]))
+        scodonre = "(?P<" + reid + ">"
+        scodonre += (
+            "["
+            + scodon[0][0]
+            + "]"
+            + "["
+            + scodon[0][1]
+            + "]"
+            + "["
+            + intersect
+            + "]"
+            + "["
+            + scodon[1][1]
+            + "]"
+            + "["
+            + scodon[1][2]
+            + "]"
+        )
     elif shift_val == 2:
-        intersect1 = ''.join(set(scodon[0][1]) & set(scodon[1][0]))
-        intersect2 = ''.join(set(scodon[0][2]) & set(scodon[1][1]))
-        scodonre = '(?P<' + reid + '>'
-        scodonre += '[' + scodon[0][0] + ']' + \
-                    '[' + intersect1 + ']' + \
-                    '[' + intersect2 + ']' + \
-                    '[' + scodon[1][2] + ']'
-    scodonre += ')'
+        intersect1 = "".join(set(scodon[0][1]) & set(scodon[1][0]))
+        intersect2 = "".join(set(scodon[0][2]) & set(scodon[1][1]))
+        scodonre = "(?P<" + reid + ">"
+        scodonre += (
+            "["
+            + scodon[0][0]
+            + "]"
+            + "["
+            + intersect1
+            + "]"
+            + "["
+            + intersect2
+            + "]"
+            + "["
+            + scodon[1][2]
+            + "]"
+        )
+    scodonre += ")"
     return scodonre
 
 
-def _get_codon_rec(pro, nucl, span_mode, alphabet, gap_char="-",
-                   codon_table=default_codon_table, complete_protein=False,
-                   max_score=10):
+def _get_codon_rec(
+    pro,
+    nucl,
+    span_mode,
+    alphabet,
+    gap_char="-",
+    codon_table=default_codon_table,
+    complete_protein=False,
+    max_score=10,
+):
     """Generate codon alignment based on regular re match (PRIVATE).
 
     span_mode is a tuple returned by _check_corr. The first element
@@ -547,45 +634,54 @@ def _get_codon_rec(pro, nucl, span_mode, alphabet, gap_char="-",
     aa2re = _get_aa_regex(codon_table)
     if mode in (0, 1):
         if len(pro.seq.ungap(gap_char)) * 3 != (span[1] - span[0]):
-            raise ValueError("Protein Record {0} and Nucleotide Record {1} "
-                             "do not match!".format(pro.id, nucl.id))
+            raise ValueError(
+                f"Protein Record {pro.id} and "
+                f"Nucleotide Record {nucl.id} do not match!"
+            )
         aa_num = 0
         for aa in pro.seq:
             if aa == "-":
                 codon_seq += "---"
             elif complete_protein and aa_num == 0:
-                this_codon = nucl_seq._data[span[0]:span[0] + 3]
-                if not re.search(_codons2re[codon_table.start_codons],
-                                 this_codon.upper()):
+                this_codon = nucl_seq._data[span[0] : span[0] + 3]
+                if not re.search(
+                    _codons2re[codon_table.start_codons], this_codon.upper()
+                ):
                     max_score -= 1
-                    warnings.warn("start codon of {0} ({1} {2}) does not "
-                                  "correspond to {3} "
-                                  "({4})".format(pro.id, aa, aa_num,
-                                                 nucl.id, this_codon),
-                                  BiopythonWarning)
+                    warnings.warn(
+                        f"start codon of {pro.id} ({aa} {aa_num}) does not "
+                        f"correspond to {nucl.id} ({this_codon})",
+                        BiopythonWarning,
+                    )
                 if max_score == 0:
-                    raise RuntimeError("max_score reached for {0}! Please "
-                                       "raise up the tolerance to get an "
-                                       "alignment in anyway".format(nucl.id))
+                    raise RuntimeError(
+                        f"max_score reached for {nucl.id}! Please raise up "
+                        "the tolerance to get an alignment in anyway"
+                    )
                 codon_seq += this_codon
                 aa_num += 1
             else:
-                this_codon = nucl_seq._data[(span[0] + 3 * aa_num):
-                                            (span[0] + 3 * (aa_num + 1))]
+                this_codon = nucl_seq._data[
+                    (span[0] + 3 * aa_num) : (span[0] + 3 * (aa_num + 1))
+                ]
                 if not str(Seq(this_codon.upper()).translate(table=codon_table)) == aa:
                     max_score -= 1
-                    warnings.warn("%s(%s %d) does not correspond to %s(%s)"
-                                  % (pro.id, aa, aa_num, nucl.id, this_codon),
-                                  BiopythonWarning)
+                    warnings.warn(
+                        "%s(%s %d) does not correspond to %s(%s)"
+                        % (pro.id, aa, aa_num, nucl.id, this_codon),
+                        BiopythonWarning,
+                    )
                 if max_score == 0:
-                    raise RuntimeError("max_score reached for {0}! Please "
-                                       "raise up the tolerance to get an "
-                                       "alignment in anyway".format(nucl.id))
+                    raise RuntimeError(
+                        f"max_score reached for {nucl.id}! Please raise up "
+                        "the tolerance to get an alignment in anyway"
+                    )
                 codon_seq += this_codon
                 aa_num += 1
         return SeqRecord(CodonSeq(codon_seq, alphabet=alphabet), id=nucl.id)
     elif mode == 2:
         from collections import deque
+
         shift_pos = deque([])
         shift_start = []
         match = span_mode[2]
@@ -599,16 +695,13 @@ def _get_codon_rec(pro, nucl, span_mode, alphabet, gap_char="-",
         while True:
             rf_table.append(i)
             i += 3
-            if i in shift_start and \
-                    m_groupdict[shift_start.index(i)].isupper():
+            if i in shift_start and m_groupdict[shift_start.index(i)].isupper():
                 shift_index = shift_start.index(i)
-                shift_val = 6 - (shift_pos[shift_index][1] -
-                                 shift_pos[shift_index][0])
+                shift_val = 6 - (shift_pos[shift_index][1] - shift_pos[shift_index][0])
                 rf_table.append(i)
                 rf_table.append(i + 3 - shift_val)
                 i = shift_pos[shift_index][1]
-            elif i in shift_start and \
-                    m_groupdict[shift_start.index(i)].islower():
+            elif i in shift_start and m_groupdict[shift_start.index(i)].islower():
                 i = shift_pos[shift_start.index(i)][1]
             if i >= match.end():
                 break
@@ -617,51 +710,62 @@ def _get_codon_rec(pro, nucl, span_mode, alphabet, gap_char="-",
             if aa == "-":
                 codon_seq += "---"
             elif complete_protein and aa_num == 0:
-                this_codon = nucl_seq._data[rf_table[0]:rf_table[0] + 3]
-                if not re.search(_codons2re[codon_table.start_codons],
-                                 this_codon.upper()):
+                this_codon = nucl_seq._data[rf_table[0] : rf_table[0] + 3]
+                if not re.search(
+                    _codons2re[codon_table.start_codons], this_codon.upper()
+                ):
                     max_score -= 1
-                    warnings.warn("start codon of {0}({1} {2}) does not "
-                                  "correspond to {3}({4})".format(
-                                      pro.id, aa, aa_num, nucl.id, this_codon),
-                                  BiopythonWarning)
+                    warnings.warn(
+                        f"start codon of {pro.id}({aa} {aa_num}) does not "
+                        f"correspond to {nucl.id}({this_codon})",
+                        BiopythonWarning,
+                    )
                     codon_seq += this_codon
                     aa_num += 1
             else:
-                if aa_num < len(pro.seq.ungap('-')) - 1 and \
-                        rf_table[aa_num + 1] - rf_table[aa_num] - 3 < 0:
+                if (
+                    aa_num < len(pro.seq.ungap("-")) - 1
+                    and rf_table[aa_num + 1] - rf_table[aa_num] - 3 < 0
+                ):
                     max_score -= 1
                     start = rf_table[aa_num]
                     end = start + (3 - shift_val)
                     ngap = shift_val
-                    this_codon = nucl_seq._data[start:end] + '-' * ngap
+                    this_codon = nucl_seq._data[start:end] + "-" * ngap
                 elif rf_table[aa_num] - rf_table[aa_num - 1] - 3 > 0:
                     max_score -= 1
                     start = rf_table[aa_num - 1] + 3
                     end = rf_table[aa_num]
                     ngap = 3 - (rf_table[aa_num] - rf_table[aa_num - 1] - 3)
-                    this_codon = (nucl_seq._data[start:end]
-                                  + '-' * ngap
-                                  + nucl_seq._data[rf_table[aa_num]:rf_table[aa_num] + 3])
+                    this_codon = (
+                        nucl_seq._data[start:end]
+                        + "-" * ngap
+                        + nucl_seq._data[rf_table[aa_num] : rf_table[aa_num] + 3]
+                    )
                 else:
                     start = rf_table[aa_num]
                     end = start + 3
                     this_codon = nucl_seq._data[start:end]
-                    if not str(Seq(this_codon.upper()).translate(table=codon_table)) == aa:
+                    if (
+                        not str(Seq(this_codon.upper()).translate(table=codon_table))
+                        == aa
+                    ):
                         max_score -= 1
-                        warnings.warn("Codon of {0}({1} {2}) does not "
-                                      "correspond to {3}({4})".format(
-                                          pro.id, aa, aa_num, nucl.id,
-                                          this_codon),
-                                      BiopythonWarning)
+                        warnings.warn(
+                            f"Codon of {pro.id}({aa} {aa_num}) does not "
+                            f"correspond to {nucl.id}({this_codon})",
+                            BiopythonWarning,
+                        )
                 if max_score == 0:
-                    raise RuntimeError("max_score reached for {0}! Please "
-                                       "raise up the tolerance to get an "
-                                       "alignment in anyway".format(nucl.id))
+                    raise RuntimeError(
+                        f"max_score reached for {nucl.id}! Please raise up "
+                        "the tolerance to get an alignment in anyway"
+                    )
                 codon_seq += this_codon
                 aa_num += 1
-        return SeqRecord(CodonSeq(codon_seq, alphabet=alphabet,
-                         rf_table=rf_table), id=nucl.id)
+        return SeqRecord(
+            CodonSeq(codon_seq, alphabet=alphabet, rf_table=rf_table), id=nucl.id
+        )
 
 
 def _align_shift_recs(recs):
@@ -672,6 +776,7 @@ def _align_shift_recs(recs):
        by a rf_table (with frameshift in some of them).
 
     """
+
     def find_next_int(k, lst):
         idx = lst.index(k)
         p = 0
@@ -679,6 +784,7 @@ def _align_shift_recs(recs):
             if isinstance(lst[idx + p], int):
                 return lst[idx + p], p
             p += 1
+
     full_rf_table_lst = [rec.seq.get_full_rf_table() for rec in recs]
     rf_num = [0] * len(recs)
     for k, rec in enumerate(recs):
@@ -686,7 +792,7 @@ def _align_shift_recs(recs):
             if isinstance(i, int):
                 rf_num[k] += 1
             # isinstance(i, float) should be True
-            elif rec.seq._data[int(i):int(i) + 3] == "---":
+            elif rec.seq._data[int(i) : int(i) + 3] == "---":
                 rf_num[k] += 1
     if len(set(rf_num)) != 1:
         raise RuntimeError("Number alignable codons unequal in given records")
@@ -701,44 +807,49 @@ def _align_shift_recs(recs):
             break
         for j, k in enumerate(col_rf_lst):
             add_lst.append((j, int(k)))
-            if isinstance(k, float) and \
-                    recs[j].seq._data[int(k):int(k) + 3] != "---":
+            if isinstance(k, float) and recs[j].seq._data[int(k) : int(k) + 3] != "---":
                 m, p = find_next_int(k, full_rf_table_lst[j])
                 if (m - k) % 3 != 0:
                     gap_num = 3 - (m - k) % 3
                 else:
                     gap_num = 0
                 if gap_num != 0:
-                    gaps = '-' * int(gap_num)
-                    seq = (recs[j].seq._data[:int(k)] + gaps
-                           + recs[j].seq._data[int(k):])
+                    gaps = "-" * int(gap_num)
+                    seq = (
+                        recs[j].seq._data[: int(k)] + gaps + recs[j].seq._data[int(k) :]
+                    )
                     full_rf_table = full_rf_table_lst[j]
                     bp = full_rf_table.index(k)
-                    full_rf_table = (full_rf_table[:bp]
-                                     + [v + int(gap_num) for v in full_rf_table[bp + 1:]])
+                    full_rf_table = full_rf_table[:bp] + [
+                        v + int(gap_num) for v in full_rf_table[bp + 1 :]
+                    ]
                     full_rf_table_lst[j] = full_rf_table
-                    recs[j].seq = CodonSeq(seq,
-                                           rf_table=recs[j].seq.rf_table,
-                                           alphabet=recs[j].seq.alphabet)
+                    recs[j].seq = CodonSeq(
+                        seq,
+                        rf_table=recs[j].seq.rf_table,
+                        alphabet=recs[j].seq.alphabet,
+                    )
                 add_lst.pop()
                 gap_num += m - k
                 i += p - 1
         if len(add_lst) != rec_num:
             for j, k in add_lst:
                 gaps = "-" * int(gap_num)
-                seq = (recs[j].seq._data[:int(k)] + gaps
-                       + recs[j].seq._data[int(k):])
+                seq = recs[j].seq._data[: int(k)] + gaps + recs[j].seq._data[int(k) :]
                 full_rf_table = full_rf_table_lst[j]
                 bp = full_rf_table.index(k)
                 inter_rf = []
                 for t in filter(lambda x: x % 3 == 0, range(len(gaps))):
                     inter_rf.append(k + t + 3.0)
-                full_rf_table = (full_rf_table[:bp] + inter_rf
-                                 + [v + int(gap_num) for v in full_rf_table[bp:]])
+                full_rf_table = (
+                    full_rf_table[:bp]
+                    + inter_rf
+                    + [v + int(gap_num) for v in full_rf_table[bp:]]
+                )
                 full_rf_table_lst[j] = full_rf_table
-                recs[j].seq = CodonSeq(seq,
-                                       rf_table=recs[j].seq.rf_table,
-                                       alphabet=recs[j].seq.alphabet)
+                recs[j].seq = CodonSeq(
+                    seq, rf_table=recs[j].seq.rf_table, alphabet=recs[j].seq.alphabet
+                )
         i += 1
     return recs
 
@@ -756,4 +867,5 @@ def _align_shift_recs(recs):
 
 if __name__ == "__main__":
     from Bio._utils import run_doctest
+
     run_doctest()
